#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
from pyromaths.outils import Arithmetique, Affichage
from pyromaths.ex import LegacyExercise
from pyromaths.ex import Jinja2Exercise
from pyromaths.outils.jinja2utils import facteur

#===============================================================================
# Poser des opérations
#===============================================================================
class PoserDesOperations(Jinja2Exercise):
    """Poser des opérations (sauf divisions)"""

    tags = ['Sixième', 'Nombres et calculs', 'Calcul posé', 'Cycle 3']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        SommeValues = self.valeurs_sommes()
        DiffValues = self.valeurs_diff()
        ProdValues = self.valeurs_prod()
        ordre = ["somme", "diff", "produit"]
        random.shuffle(ordre)
        self.context = {
            "ordre": ordre,
            "Sna": SommeValues[0],
            "Sta": SommeValues[1],
            "Snb": SommeValues[2],
            "Stb": SommeValues[3],
            "Str": SommeValues[4],
            "Stt": SommeValues[5],
            "Dna": DiffValues[0],
            "Dta": DiffValues[1],
            "Dnb": DiffValues[2],
            "Dtb": DiffValues[3],
            "Dtrh": DiffValues[4],
            "Dtrb": DiffValues[5],
            "Dtt": DiffValues[6],
            "Pna": ProdValues[0],
            "Pnb": ProdValues[1],
            "Ppa": ProdValues[2],
            "Ppb": ProdValues[3],
            "alignement": ProdValues[4],
        }


    @property
    def environment(self):
        environment = super().environment
        environment.filters.update({
            'facteur': facteur,
            })
        return environment

    def splitDecimal(self, nb):
        """
        Retourne une liste contenant la partie entière puis la partie décimale d'un nombre
        :param nb: int or Decimal
        :return: list
        """
        s =format(nb, ".14g") # Convertit un float en un décimal
        try:
            p = s.index('.')
            l = [s[:p], s[p+1:]]
        except ValueError:
            "Il s'agit d'un entier"
            l=[s, '']
        return l

    def valeurs_sommes(self):
        """
        Sélectionne un nombre entier et un nombre décimal non entier pour poser une somme
        :return:
            Sna, Snb: int or float
            Sta, Stb, Stt, Str: justified strings
        """
        Sna = random.randrange(111, 100000) # Nombre entier
        Snb = (random.randrange(11, 10000)*10+random.randrange(1,10)) / 10**random.randrange(1,3) # Nombre décimal non entier
        la = self.splitDecimal(Sna)
        lb = self.splitDecimal(Snb)
        lt = self.splitDecimal(Sna + Snb)
        "Justifie les nombres sous forme de texte pour que les virgules soient alignées"
        alignement = '{:>' + str(len(lt[0])) + '}.{:0>' +str(max(len(la[1]), len(lb[1]))) + '}'
        Sta = alignement.format(la[0], la[1])
        Stb = alignement.format(lb[0], lb[1])
        Stt = alignement.format(lt[0], lt[1])
        Str = self.retenues_somme(Sta, Stb)
        "Mélange l'ordre des deux valeurs"
        if random.randrange(2):
            return Sna, Sta, Snb, Stb, Str, Stt
        else:
            return Snb, Stb, Sna, Sta, Str, Stt

    def valeurs_diff(self):
        """
        Sélectionne un nombre entier et un nombre décimal non entier pour poser une différence
        :return:
            Sna, Snb: int or float
            Sta, Stb, Stt, Strb, Strh: justified strings
        """
        Dna = random.randrange(111, 100000) # Nombre entier
        Dnb = (random.randrange(11, 10000)*10+random.randrange(1,10)) / 10**random.randrange(1,3) # Nombre décimal non entier
        if Dna<Dnb: Dna, Dnb = Dnb, Dna
        la = self.splitDecimal(Dna)
        lb = self.splitDecimal(Dnb)
        lt = self.splitDecimal(Dna - Dnb)
        "Justifie les nombres sous forme de texte pour que les virgules soient alignées"
        alignement = '{:>' + str(len(la[0])) + '}.{:0>' +str(max(len(la[1]), len(lb[1]))) + '}'
        Dta = alignement.format(la[0], la[1])
        Dtb = alignement.format(lb[0], lb[1])
        Dtt = alignement.format(lt[0], lt[1])
        Dtrh, Dtrb = self.retenues_diff(Dta, Dtb)
        return Dna, Dta, Dnb, Dtb, Dtrh, Dtrb, Dtt

    def valeurs_prod(self):
        Pna = random.randrange(10, 1000)*10+random.randrange(1,10)
        Pnb = random.randrange(10, 100)*10+random.randrange(1,10)
        Pnt = format(Pna * Pnb, ".14g")
        alignement = '{:>' + str(len(Pnt)) + '}'
        Ppa = random.randrange(-3,0) # Position de la virgule
        Ppb = random.randrange(-3,0)
        if random.randrange(2):
            return Pna, Pnb, Ppa, Ppb, alignement
        else:
            return Pnb, Pna, Ppb, Ppa, alignement

    def spaceToZeros(self, t):
        "Convertit un caracatère en entier, renvoie 0 si ce n'est pas un chiffre"
        try:
            return int(t)
        except ValueError:
            return 0

    def retenues_somme(self, ta, tb):
        tr=' ' # liste des retenues
        ra, rb=ta[::-1], tb[::-1] # retourne le nombre : '123.45' -> '54.321'
        for i in range(len(ta)-1):
            tr+=str((self.spaceToZeros(tr[i]) + self.spaceToZeros(ra[i]) + self.spaceToZeros(rb[i]))//10).replace('0', ' ')
        return tr[::-1]

    def retenues_diff(self, ta, tb):
        trh='' # liste des retenues hautes
        trb=' ' # liste des retenues basses
        ra, rb=ta[::-1], tb[::-1] # retourne le nombre : '123.45' -> '54.321'
        for i in range(len(ta)-1):
            if self.spaceToZeros(ra[i])<self.spaceToZeros(rb[i])+self.spaceToZeros(trb[i]):
                trh+='1'
                trb+='1'
            else:
                trh+=' '
                trb+=' '
        trh+=' '
        return trh[::-1], trb[::-1]



#===============================================================================
# Calcul mental
#===============================================================================


def tex_calcul_mental(exo, cor):
    modules = (plus, moins, plus, div)
    calculs = [i for i in range(20)]
    for i in range(20):
        j = random.randrange(0, len(calculs))
        (a, b) = modules[calculs[j] // 5](10)
        if calculs[j] // 5 == 0:
            choix_trou(a, b, a + b, '+', exo, cor)
        if calculs[j] // 5 == 1:
            choix_trou(a, b, a - b, '-', exo, cor)
        if calculs[j] // 5 == 2:
            choix_trou(a, b, a * b, '\\times', exo, cor)
        if calculs[j] // 5 == 3:
            choix_trou(a, b, a // b, '\\div', exo, cor)
        calculs.pop(j)


def choix_trou(nb1, nb2, tot, operateur, exo, cor):
    nbaleatoire = random.randrange(4)
    if nbaleatoire > 1:
        exo.append('\\item $%s %s %s = \\ldots\\ldots$' % (nb1,
                 operateur, nb2))
        cor.append('\\item $%s %s %s = \\mathbf{%s}$' % (nb1,
                 operateur, nb2, tot))
    elif nbaleatoire > 0:
        exo.append('\\item $%s %s \\ldots\\ldots = %s$' % (nb1,
                 operateur, tot))
        cor.append('\\item $%s %s \\mathbf{%s} = %s$' % (nb1,
                 operateur, nb2, tot))
    else:
        exo.append('\\item $\\ldots\\ldots %s %s = %s$' % (operateur,
                 nb2, tot))
        cor.append('\\item $\\mathbf{%s} %s %s = %s$' % (nb1,
                 operateur, nb2, tot))


def plus(valeurmax):
    (a, b) = (Arithmetique.valeur_alea(1, valeurmax), Arithmetique.valeur_alea(1,
              valeurmax))
    return (a, b)


def moins(valeurmax):
    (a, b) = (Arithmetique.valeur_alea(1, valeurmax), Arithmetique.valeur_alea(1,
              valeurmax))
    return (a + b, a)


def div(valeurmax):
    (a, b) = (Arithmetique.valeur_alea(1, valeurmax), Arithmetique.valeur_alea(1,
              valeurmax))
    return (a * b, a)




def _CalculMental():
    exo = ["\\exercice", 'Effectuer sans calculatrice :', '\\begin{multicols}{4}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*", 'Effectuer sans calculatrice :', '\\begin{multicols}{4}\\noindent', '\\begin{enumerate}']

    tex_calcul_mental(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)

class CalculMental(LegacyExercise):
    """Calcul mental"""

    tags = ["Sixième"]
    function = _CalculMental


#===============================================================================
# PRODUITS ET QUOTIENTS PAR 10, 100, 1000
#===============================================================================


def tex_dix(exo, cor):
    nb = 4  # nb de calculs de chaque type
    l = valeurs10(nb)
    for dummy in range(len(l)):
        j = random.randrange(0, len(l))
        tex_formule_dix(l.pop(j), exo, cor)


def tex_formule_dix(l, exo, cor):
    if l[2] == '*':
        alea = random.randrange(0, 5)
        if alea > 1:
            exo.append('\\item $%s \\quad\\times\\quad %s \\quad = \\quad \\dotfill$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1], 1)))
            cor.append('\\item $%s \\times %s = \\mathbf{%s}$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] * l[1], 1)))
        elif alea > 0:
            exo.append('\\item $%s \\quad\\times\\quad \\dotfill \\quad = \\quad %s$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[0] * l[1], 1)))
            cor.append('\\item $%s \\times \\mathbf{%s} = %s$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] * l[1], 1)))
        else:
            exo.append('\\item $\\dotfill \\quad\\times\\quad %s \\quad = \\quad %s$' %
                       (Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] * l[1], 1)))
            cor.append('\\item $\\mathbf{%s} \\times %s = %s$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] * l[1], 1)))
    else:
        alea = random.randrange(0, 5)
        if alea > 1:
            exo.append('\\item $%s \\quad\\div\\quad %s \\quad = \\quad \\dotfill$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[1], 1)))
            cor.append('\\item $%s \\div %s = \\mathbf{%s}$' % (Affichage.decimaux(l[0],1), Affichage.decimaux(l[1], 1),
                                                                Affichage.decimaux(l[0] / l[1], 1)))
        elif alea > 0:
            exo.append('\\item $%s \\quad\\div\\quad \\dotfill \\quad = \\quad %s$' %
                       (Affichage.decimaux(l[0], 1), Affichage.decimaux(l[0] / l[1], 1)))
            cor.append('\\item $%s \\div \\mathbf{%s} = %s$' % (Affichage.decimaux(l[0], 1),
                                                                Affichage.decimaux(l[1], 1),
                                                                Affichage.decimaux(l[0] / l[1], 1)))
        else:
            exo.append('\\item $\\dotfill \\quad\\div\\quad %s \\quad = \\quad %s$' %
                       (Affichage.decimaux(l[1], 1), Affichage.decimaux(l[0] / l[1], 1)))
            cor.append('\\item $\\mathbf{%s} \\div %s = %s$' % (Affichage.decimaux(l[0], 1),
                                                                Affichage.decimaux(l[1], 1),
                                                                Affichage.decimaux(l[0] / l[1], 1)))


def valeurs10(nb):  # renvoie nb valeur de chaque type : *10, /10, *0.1
    l = []
    for i in range(nb):
        if random.randrange(0, 1):
            l.append((Arithmetique.valeur_alea(111, 999) * 10 ** random.randrange(-3, 0), 10 ** (i + 1), '*'))
        else:
            l.append((10 ** (i + 1), Arithmetique.valeur_alea(111, 999) * 10 **random.randrange(-3, 0), '*'))
    for i in range(nb):
        l.append((Arithmetique.valeur_alea(111, 999) * 10 ** random.randrange(-3, 0), 10 ** (i + 1), '/'))
    for i in range(nb):
        if random.randrange(0, 1):
            l.append((Arithmetique.valeur_alea(111, 999) * 10 ** random.randrange(-3, 0), 10 ** (-i - 1), '*'))
        else:
            l.append((10 ** (-i - 1), Arithmetique.valeur_alea(111, 999) * 10 **random.randrange(-3, 0), '*'))
    return l



def _ProduitPuissanceDix():
    exo = ["\\exercice", u'Compléter sans calculatrice :', '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*", u'Compléter sans calculatrice :', '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']

    tex_dix(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)

class ProduitPuissanceDix(LegacyExercise):
    """Produits, quotients par 10, 100, 1000"""

    tags = ["Sixième"]
    function = _ProduitPuissanceDix



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
from pyromaths.ex import LegacyExercise


def valeurs_diviseurs():
    diviseurs = [2, 3, 5, 9, 10]
    liste = []
    for dummy in range(5):
        liste.append(diviseurs.pop(random.randrange(len(diviseurs))) *
                     random.randrange(11, 100))
    return liste


def liste_diviseurs(l):
    diviseurs = (2, 3, 5, 9, 10)
    reponse = []
    for i in range(len(l)):
        reponse.extend([[l[i]]])
        for j in range(len(diviseurs)):
            if l[i] % diviseurs[j]:  # n'est pas divisible
                reponse[i].append("$\\square$")
            else:
                reponse[i].append("$\\checkedbox$")
    return reponse


def _Divisible():
    l = valeurs_diviseurs()
    reponse = liste_diviseurs(l)

    exo = ["\\exercice", u'Cocher les bonnes réponses :\\par']
    cor = ["\\exercice*", u'Cocher les bonnes réponses :\\par']

    exo.append("\\begin{tabular}{c@{ est divisible : \kern1cm}l@{ par 2\\kern1cm}l@{ par 3\\kern1cm}l@{ par 5\\kern1cm}l@{ par 9\\kern1cm}l@{ par 10}}")
    cor.append("\\begin{tabular}{c@{ est divisible : \kern1cm}l@{ par 2\\kern1cm}l@{ par 3\\kern1cm}l@{ par 5\\kern1cm}l@{ par 9\\kern1cm}l@{ par 10}}")
    for i in range(len(l)):
        exo.append("%s & $\\square$ & $\\square$ & $\\square$ & $\\square$ & $\\square$ \\\\" %
                 l[i])
        cor.append("%s & %s & %s & %s & %s & %s \\\\" % tuple(reponse[i]))
    exo.append("\\end{tabular}")
    cor.append("\\end{tabular}")
    return (exo, cor)

class Divisible(LegacyExercise):
    """Multiples de 2, 3, 5, 9, 10"""

    tags = ["Sixième"]
    function = _Divisible



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils.Arithmetique import pgcd, ppcm
from collections import Counter
from pyromaths.classes.SquareRoot import SquareRoot
from pyromaths.outils.Priorites3 import EstNombre, texify
from functools import reduce

class Fraction:
    """Cette classe crée la notion de fractions.
    code permet de préciser si une décomposition a pour objectif une mise au
    même dénominateur 'r' ou une simplification 's'

    >>> from pyromaths.classes.Fractions import Fraction
    >>> repr(Fraction(5,6))
    Fraction(5, 6)
    >>> repr(Fraction('x',6))
    Fraction("x", 6)
    >>> repr(Fraction(3.4))
    Fraction(34, 10)
    >>> repr(Fraction(3.0, 4.0))
    Fraction(3, 4)
    >>> repr(Fraction(Fraction(1,3)))
    Fraction(1, 3)

    """
    def __init__(self, n, d=1, code=""):
        from pyromaths.classes.PolynomesCollege import Polynome
        if isinstance(n, float) and abs(n) != float("inf"):
            e = len(format(float(n), ".15g").partition('.')[2])
            n = int(n * 10 ** e)
            d = d * 10 ** e
        if isinstance(d, float):
            e = len(str(d).partition('.')[2].rstrip('0'))
            d = int(d * 10 ** e)
            n = n * 10 ** e
        self.n = n
        if n == 0 :
            self.d = 1
        else:
            self.d = d
        self.code = code
        if isinstance(n, Fraction) and d == 1 and code == "":
            # Fraction(Fraction(1, 3)) renvoie Fraction(1, 3)
            self.n = n.n
            self.d = n.d
            self.code = n.code


    def __str__(self):
        r"""**str**\ (*object*)

        Renvoie une version LaTeX de la :class:`Fraction`.
            >>> from pyromaths.classes.Fractions import Fraction
            >>> str(Fraction(8,1))
            8
            >>> str(Fraction(5,6))
            \dfrac{5}{6}
            >>> str(Fraction('-5*2', '3*2', 'r'))
            \dfrac{-5_{\times 2}}{3_{\times 2}}
            >>> str(Fraction('5*-7*2', '11*2*5', 's'))
            \dfrac{\cancel{5}\times \left( -7\right) \times \cancel{2}}{11\times \cancel{2}\times \cancel{5}}
            >>> str(Fraction('-144', '22', 's'))
            \dfrac{-72\times \cancel{2}}{11\times \cancel{2}}
            >>> from pyromaths.classes.SquareRoot import SquareRoot
            >>> str(Fraction(SquareRoot([[-10, None], [-1, 80]]), -2))
            \dfrac{-10-\sqrt{80}}{-2}
            >>> str(Fraction(SquareRoot([[-10, None], [-4, 5]]), -2, 's'))
            \dfrac{\left( 5+2\,\sqrt{5}\right) \times \cancel{-2}}{1\times \cancel{-2}}

        :rtype: string
        """
        from pyromaths.outils.Priorites3 import splitting
        #=======================================================================
        # print (repr(self))
        #=======================================================================
        if self.n == 0 or self.n == '0':
            return '0'
        lnum, lden = [], []
        if isinstance(self.n, str): lnum = splitting(self.n)
        if isinstance(self.d, str): lden = splitting(self.d)
        if len(lnum) == 1: self.n, lnum = eval(self.n), []
        if len(lden) == 1: self.d, lden = eval(self.d), []
        if self.d == 1:
            return (str(self.n), texify([lnum])[0])[lnum != []]
        if self.code == "r":
            # lnum et lden doivent être définis et se terminer par [..., '*', valeur]
            if not lnum or not lden or lnum[-2:] != lden[-2:]:
                raise ValueError(u'Mauvais usage de l\'étiquettes "r" dans %r' % self)
            lden[-2], lden[-1], lnum[-2], lnum[-1] = [lden[-2], 'indice'], [lden[-1], 'indice'], [lnum[-2], 'indice'], [lnum[-1], 'indice']
            return '\\dfrac{%s}{%s}' % (texify([lnum])[0], texify([lden])[0])
        if self.code == "s":
            if isinstance(self.n, (int, float, SquareRoot)) and isinstance(self.d, (int, float, SquareRoot)):
                lepgcd = pgcd(self.n, self.d)
                lnum, lden = [repr(self.n // lepgcd), '*', [str(lepgcd), 'cancel']], [repr(self.d // lepgcd), '*', [str(lepgcd), 'cancel']]
            else:
                for i in range(len(lnum)):
                    if lnum[i] != '*' and lden.count(lnum[i]):
                        # On doit simplifier la fraction par ce nombre
                        j = lden.index(lnum[i])
                        lden[j] = [lden[j], 'cancel']
                        lnum[i] = [lnum[i], 'cancel']
            return '\\dfrac{%s}{%s}' % (texify([lnum])[0], texify([lden])[0])
        s = r'\dfrac'
        s += '{%s}' % (self.n, texify([lnum])[0])[lnum != []]
        s += '{%s}' % (self.d, texify([lden])[0])[lden != []]
        return s


    def __repr__(self):
        """**repr**\ (*object*)

        Renvoie une chaîne de caractère représentant une :mod:`Fraction`
        évaluable pour créer un :mod:`Fraction`.

            >>> from pyromaths.classes.Fractions import Fraction
            >>> repr(Fraction(5,6))
            Fraction(5, 6)
            >>> repr(Fraction('-72*2', '11*2', 'r'))
            Fraction("-72*2", "11*2", "r")
            >>> repr(Fraction('-72*2', '11*2', 's'))
            Fraction("-72*2", "11*2", "s")

        :rtype: string
        """
        if isinstance(self.n, str):
            num = "\"%s\"" % self.n
        else:
            num = "%r" % self.n
        if isinstance(self.d, str):
            den = "\"%s\"" % self.d
        else:
            den = "%r" % self.d
        if self.code:
            return "Fraction(%s, %s, \"%s\")" % (num, den, self.code)
        else:
            return "Fraction(%s, %s)" % (num, den)

    def __abs__(self):
        """Renvoie la valeur absolue d'une fraction
        """
        self = Fraction(self.n, self.d)
        if self.n * self.d > 0:
            return self
        else:
            return -self

    def __add__(self, *others):
        """*object*\ .\ **__add__**\ (*other*)

        ``p.__add__(q)`` est équivalent à ``p + q``  calcule la somme de deux fractions.

        *other* peut être une chaîne représentant une fraction, un entier ou un réel.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> Fraction(2,5) + Fraction(2,10)
        Fraction("2*2", "5*2", "r")+Fraction(2, 10)
        >>> Fraction(2,20) + Fraction(2,10)
        Fraction(2, 20)+Fraction("2*2", "10*2", "r")
        >>> repr(Fraction(5,10) + Fraction(2,10))
        Fraction(7, 10)
        >>> Fraction(5,7) + Fraction(2,10)
        Fraction("5*10", "7*10", "r")+Fraction("2*7", "10*7", "r")

        :param: other
        :type: Fraction ou string
        :rtype: Fraction ou string

        **TODO :** Attention, 1+3/4 donne 1*4/1*4 + 3/4 à la place de 4/4+3/4. À corriger
        """
        from pyromaths.classes.PolynomesCollege import Polynome
        lother, lden, traiter = [], [], False
        if self.code: traiter = True
        details, var = 0, ''
        for other in others:
            if other == 0:
                pass
            elif isinstance(other, Polynome):
                var = other.var
                if details != 0  :
                    details = min(details, other.details)
                else: details = other.details
                lother.append(other)
            elif isinstance(other, (int, float)):
                    lother.append(Fraction(other))
                    lden.append(1)
            elif isinstance(other, Fraction):
                lother.append(other)
                if other.code: traiter = True
                try: lden.append(eval(other.d))
                except TypeError: lden.append(other.d)
            else:
                raise ValueError(u'Format incorrect : %s' % (other))
        if var:
            self = Polynome([[self, 0]], var, details)
            for i in range(len(lother)):
                if not isinstance(lother[i], Polynome): lother[i] = Polynome([[lother[i], 0]], var, details)
            return Polynome.__add__(self, *lother)

        if traiter:
            lfrac = [repr(self.traitement())]
            for other in lother:
                lfrac.append(repr(other.traitement()))
            return "+".join(lfrac)
        try: self.d = eval(self.d)
        except TypeError: pass
        if not lother: return self  # On a ajouté 0
        leppcm = ppcm(self.d, *lden)
        if self.d == leppcm:
            # Vérifions si toutes les fractions ont le même dénominateur
            d = Counter(lden)
            if d[leppcm] == len(lden):
                try: num = eval(self.n)
                except TypeError: num = self.n
                for other in lother:
                    try: num += eval(other.n)
                    except TypeError: num += other.n
                return Fraction(num, leppcm)
        if self.n: lfrac = [repr(self.choix_denominateur(leppcm))]
        else: lfrac = []
        for other in lother: lfrac.append(repr(other.choix_denominateur(leppcm)))
        if lfrac: return "+".join(lfrac)
        else: return "0"

    def choix_denominateur(self, denominateur):
        """
        Écrit la fraction self avec le dénominateur denominateur
        """
        if denominateur != self.d:
            return Fraction("%s*%s" % (self.n, denominateur // self.d), "%s*%s" % (self.d, denominateur // self.d), "r")
        else:
            return self

    def __radd__(self, other):
        """*object*\ .\ **__radd__**\ (*other*)

        ``p.__radd__(q)`` est équivalent à ``p + q``  calcule la somme de l'objet p avec la fraction q

        *other* peut être une chaîne représentant un entier ou un réel.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> 3 + Fraction(2,5)
        Fraction("3*5", "1*5", "r")+Fraction(2, 5)

        :param: other
        :type: real ou integer
        :rtype: string
        """
        if other == 0 : return self
        if self == 0 : return other
        if isinstance(other, str):
            other = eval(other)
            if isinstance(other, str): return '%s+%r' % (other, self)
            else: return '%r+%r' % (other, self)
        return Fraction(other) + self

    def __sub__(self, *others):
        """*object*\ .\ **__sub__**\ (*other*)

        ``p.__sub__(q)`` est équivalent à ``p - q``  calcule la différence de deux fractions.

        *other* peut être une chaîne représentant une fraction, un entier ou un réel.

        Pour plus de détails, voir :py:func:`__add__`"""
        from pyromaths.classes.PolynomesCollege import Polynome
        lother, lden, traiter = [], [], False
        if self.code: traiter = True
        details, var = True, ''
        for other in others:
            if other == 0:
                pass
            elif isinstance(other, Polynome):
                var = other.var
                if details != 0 : details = min(details, other.details)
                else: details = other.details
                lother.append(other)
            elif isinstance(other, (int, float)):
                    lother.append(Fraction(other))
                    lden.append(1)
            elif isinstance(other, Fraction):
                lother.append(other)
                if other.code: traiter = True
                try: lden.append(eval(other.d))
                except TypeError: lden.append(other.d)
            else:
                raise ValueError(u'Format incorrect : %s' % (other))
        if var:
            self = Polynome([[self, 0]], var, details)
            for i in range(len(lother)):
                if not isinstance(lother[i], Polynome): lother[i] = Polynome([[lother[i], 0]], var, details)
            return Polynome.__sub__(self, *lother)

        if traiter:
            lfrac = [repr(self.traitement())]
            for other in lother:
                lfrac.append(repr(other.traitement()))
            return "-".join(lfrac)
        try: self.d = eval(self.d)
        except TypeError: pass
        if not lother: return self  # On a ajouté 0
        leppcm = ppcm(self.d, *lden)
        if self.d == leppcm:
            # Vérifions si toutes les fractions ont le même dénominateur
            d = Counter(lden)
            if d[leppcm] == len(lden):
                try: num = eval(self.n)
                except TypeError: num = self.n
                for other in lother:
                    try: num += -eval(other.n)
                    except TypeError: num += -other.n
                return Fraction(num, leppcm)
        if self.n: lfrac = [repr(self.choix_denominateur(leppcm))]
        else: lfrac = []
        for other in lother:
            if lfrac: lfrac.append(repr(other.choix_denominateur(leppcm)))
            else: lfrac.append(repr(-other.choix_denominateur(leppcm)))
            # On effectue Fraction(0,3)-Fraction(2,3), il faut donc prendre l'opposé de Fraction(2,3), puisque Fraction(0,3) a été supprimée
        if lfrac: return "-".join(lfrac)
        else: return "0"

    def __rsub__(self, other):
        """*object*\ .\ **__rsub__**\ (*other*)

        ``p.__rsub__(q)`` est équivalent à ``p - q``  calcule la différence de l'objet p par la fraction q.

        *other* peut être un entier ou un réel.

        Pour plus de détails, voir :py:func:`__radd__`"""
        return Fraction(other) - self

    def __mul__(self, *others):
        """*object*\ .\ **__mul__**\ (*other*)

        ``p.__mul__(q)`` est équivalent à ``p * q``  calcule le produit deux fractions.

        *other* peut être une chaîne représentant une fraction, un entier ou un réel.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(Fraction(2,5) * Fraction(2,10))
        Fraction("2*2", "5*2*5", "s")
        >>> repr(Fraction(2,5) * 4)
        Fraction(8, 5)
        >>> repr(Fraction(63,20) * Fraction(8,27))
        Fraction("9*7*4*2", "4*5*9*3", "s")
        >>> repr(Fraction(24,12) * 12)
        Fraction("24*12", "12*1", "s")
        >>> repr(12*Fraction(24,12))
        Fraction("12*24", "1*12", "s")

        :param: other
        :type: Fraction ou string
        :rtype: Fraction ou string
        """
        from pyromaths.classes.PolynomesCollege import Polynome
        from pyromaths.classes.Polynome import Polynome as PolynomeLycee
        from .Racine import RacineDegre2

        if self == 0: return 0
        self = Fraction(self.n, self.d)  # Pour contourner le cas où self = Fraction(Fraction(1, 1), 1)
        lother, lnum, lden, details, var, traiter, lycee = [self], [self.n], [self.d], 0, '', False, False
        for other in others:
            if other == 0:
                return 0
            elif isinstance(other, Polynome):
                var = other.var
                details = min(details, other.details)
                lother.append(other)
            elif isinstance(other, PolynomeLycee):
                var = other.var
                lycee = True
                lother.append(other)
            elif isinstance(other, (int, float)):
                lden.append(1)
                lnum.append(other)
                lother.append(Fraction(other))
            elif isinstance(other, Fraction):
                lother.append(other)
                if other.code: traiter = True
                try: lden.append(eval(other.d))
                except TypeError: lden.append(other.d)
                try: lnum.append(eval(other.n))
                except TypeError: lnum.append(other.n)
            elif isinstance(other, RacineDegre2):
                lother.append(other)
                lden.append(other.denominateur)
                lnum.append(other.numerateur)
            else:
                raise ValueError(u'Format incorrect : %s' % (other))
        if var:
            if lycee:
                self = PolynomeLycee({0:self}, var)
                return PolynomeLycee.__mul__(self, other)
            else:
                self = Polynome([[self, 0]], var, details)
                return Polynome.__mul__(self, *lother[1:])

        if traiter:
            lfrac = [repr(self.traitement())]
            for other in lother:
                lfrac.append(repr(other.traitement()))
            return "*".join(lfrac)
        try: self.d = eval(self.d)
        except TypeError: pass
        s = abs(pgcd(reduce(lambda x, y: x * y, lnum), reduce(lambda x, y: x * y, lden)))
        if s == 1: return Fraction(reduce(lambda x, y: x * y, lnum), reduce(lambda x, y: x * y, lden))
        lepgcd, num, den = s, [], []
        i = 0
        while i < len(lnum):
            if lepgcd == 1 or lepgcd in lnum:
                num.append("*".join([repr(lnum[k]) for k in range(i, len(lnum))]))
                break
            else:
                lepgcdtmp = pgcd(lnum[i], lepgcd)
                if lepgcdtmp != 1: num.append("%r*%r" % (lepgcdtmp, lnum[i] // lepgcdtmp))
                else: num.append("%r" % lnum[i])
                lepgcd = lepgcd // lepgcdtmp
                i += 1
        i, lepgcd = 0, s
        while i < len(lden):
            if lepgcd == 1 or lepgcd in lden:
                den.append("*".join([repr(lden[k]) for k in range(i, len(lden))]))
                break
            else:
                lepgcdtmp = pgcd(lden[i], lepgcd)
                if lepgcdtmp != 1: den.append("%r*%r" % (lepgcdtmp, lden[i] // lepgcdtmp))
                else: den.append("%r" % lden[i])
                lepgcd = lepgcd // lepgcdtmp
                i += 1
        num, den = "*".join(num), "*".join(den)
        while num[:2] == "1*": num = num[2:]
        while "*1*" in num: num = num.replace("*1*", "*", 1)
        if num[-2:] == "*1": num = num[:-2]
        if s == reduce(lambda x, y: x * y, lnum):
            # laisser un *1 ou 1* selon
            if self.n == 1: num = "1*" + num
            else: num += "*1"
        while den[:2] == "1*": den = den[2:]
        while "*1*" in den: den = den.replace("*1*", "*" , 1)
        if den[-2:] == "*1": den = den[:-2]
        if s == reduce(lambda x, y: x * y, lden):
            # laisser un *1
            if self.d == 1: den = "1*" + den
            else: den += "*1"

        if "*" in num: num = num.strip()
        if "*" in den: den = den.strip()
        return Fraction(num, den, "s")


    def __rmul__(self, other):
        """*object*\ .\ **__rmul__**\ (*other*)

        ``p.__rmul__(q)`` est équivalent à ``p * q``  calcule le produit de l'objet p par la fraction q.

        *other* peut être un entier ou un réel.

        Pour plus de détails, voir :py:func:`__radd__`"""
        return Fraction(other) * self

    # def __truediv__(self, fraction): # pour Python 3
    def __truediv__(self, other):
        """*object*\ .\ **__div__**\ (*other*)

        ``p.__div__(q)`` est équivalent à ``p / q``  calcule le quotient de deux fractions.

        *other* peut être une chaîne représentant une fraction, un entier ou un réel.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(Fraction(2,5) / Fraction(10,2))
        'Fraction(2, 5)*Fraction(2, 10)'
        >>> repr(Fraction(2,5) / 4)
        'Fraction(2, 5)*Fraction(1, 4)'

        :param: other
        :type: Fraction ou string
        :rtype: Fraction ou string
        """
        if (isinstance(other, int) or isinstance(other, float)):
            other = Fraction(other)
        return "%r*%r" % (self, ~other)

    def __rtruediv__(self, other):
        """*object*\ .\ **__rtruediv__**\ (*other*)

        ``p.__rtruediv__(q)`` est équivalent à ``p / q``  calcule le produit de l'objet p par la fraction q.

        *other* peut être un entier ou un réel.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(1/Fraction(1,3))
        '1*Fraction(3, 1)'

        Pour plus de détails, voir :py:func:`__radd__`"""
        return "%r*%r" % (other, ~self)

    def __invert__(self):
        """**__invert__**\ (*self*)

        ``__invert__(p)`` est équivalent à ``~p`` calcule l'inverse de la fraction p.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(~Fraction(8,27))
        Fraction(27, 8)
        """
        return Fraction(self.d, self.n)

    def __neg__(self):
        """**__neg__**\ (*self*)

        ``__neg__(p)`` est équivalent à ``-p`` calcule l'opposé de la fraction p.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(-Fraction(8,27))
        Fraction(-8, 27)
        """
        if self.code: self = self.traitement()
        return Fraction(-self.n, self.d)

    def __pos__(self):
        """**__pos__**\ (*self*)

        ``__pos__(p)`` est équivalent à ``+p`` Renvoie la fraction p.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(+Fraction(8,27))
        Fraction(8, 27)
        """
        return Fraction(self.n, self.d)

    def __pow__(self, n):
        """**__pow__**\ (*self*, *n*)

        ``p__pow__(q)`` est équivalent à ``p**q`` calcule p à la puissance q.

        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(Fraction(2,3)**4)
        Fraction(16, 81)

        :param: n
        :type: Integer
        :rtype: Fraction
        """
        return Fraction(self.n ** n, self.d ** n)

    def __trunc__(self):
        return self.n // self.d

    def __lt__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            other = Fraction(other)
        if other.traitement().d * self.traitement().d > 0:
            return self.traitement().n * other.traitement().d < self.traitement().d * other.traitement().n
        else :
            return not(self.traitement().n * other.traitement().d < self.traitement().d * other.traitement().n)

    def __le__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            other = Fraction(other)
        if other.d * self.d > 0:
            return self.n * other.d <= self.d * other.n
        else :
            return not(self.n * other.d <= self.d * other.n)

    def __eq__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            other = Fraction(other)
        if isinstance(other, Fraction):
            return self.reduit().n * other.reduit().d == self.reduit().d * other.reduit().n

    def __ne__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            other = Fraction(other)
        return self.n * other.d != self.d * other.n

    def __gt__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            other = Fraction(other)
        if other.d * self.d > 0:
            return self.n * other.d > self.d * other.n
        else :
            return not(self.n * other.d > self.d * other.n)

    def __ge__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            other = Fraction(other)
        if other.d * self.d > 0:
            return self.n * other.d >= self.d * other.n
        else :
            return not(self.n * other.d >= self.d * other.n)

    def __float__(self):
        f = self.traitement()
        return f.n/f.d

    def __int__(self):
        assert self.n % self.d == 0, "La fraction n'est pas un nombre entier !"
        return int(self.n / self.d)

    def reduit(self):
        """**reduit**\ (*object*)

        Retourne une version réduite de la fraction, ie calcule le numérateur et le dénominateur

            >>> from pyromaths.classes.Fractions import Fraction
            >>> repr(Fraction.reduit(Fraction(2*4,5*4)))
            Fraction(8, 20)
            >>> repr(Fraction.reduit(Fraction('2*4', '5*4')))
            Fraction(8, 20)

        :param type: Fraction
        :rtype: Fraction
        """
        if isinstance(self.n, str): self.n = eval(self.n)
        if isinstance(self.d, str): self.d = eval(self.d)
        return Fraction(self.n, self.d)

    def simplifie(self):
        """**simplifie**\ (*object*)

        Retourne une version irréductible de la fraction

            >>> from pyromaths.classes.Fractions import Fraction
            >>> repr(Fraction.simplifie(Fraction(2*4,5*4)))
            Fraction(2, 5)
            >>> repr(Fraction.simplifie(Fraction('2*4','5*4')))
            Fraction(2, 5)

        :param type: Fraction
        :rtype: Fraction
        """
        conv = False
        if isinstance(self.n, str):
            self.n = eval(self.n)
            conv = True
        if isinstance(self.d, str):
            self.d = eval(self.d)
            conv = True
        if conv : self = Fraction(self.n, self.d)
        conv = False
        if isinstance(self.n, SquareRoot):
            s = self.n.simplifie()
            if s != self.n:
                self.n = s
                conv = True
        if isinstance(self.d, SquareRoot):
            s = self.d.simplifie()
            if s != self.d:
                self.d = s
                conv = True
        if conv: return self
        s = pgcd(self.n, self.d)
        if self.d != s:
            return Fraction(self.n // s, self.d // s)
        else:
            return self.n // s

    def decompose(self):
        """**decompose**\ (*object*)

        Retourne une décomposition de la fraction afin de la simplifier

            >>> from pyromaths.classes.Fractions import Fraction
            >>> repr(Fraction.decompose(Fraction(8,20)))
            Fraction("2*4", "5*4", "s")

        :param type: Fraction
        :rtype: Fraction
        """
        if isinstance(self.n, str): self.n = eval(self.n)
        if isinstance(self.d, str): self.d = eval(self.d)
        conv = False
        if isinstance(self.n, SquareRoot):
            s = self.n.simplifie()
            if s != self.n:
                self.n = s
                conv = True
        if isinstance(self.d, SquareRoot):
            s = self.d.simplifie()
            if s != self.d:
                self.d = s
                conv = True
        if conv:
            return self
        if self.d == self.n:
            return 1
        else:
            lepgcd = pgcd(self.n, self.d)
            if lepgcd == 1:
                return self
            else:
                return Fraction("%r*%s" % (self.n // lepgcd, lepgcd), "%r*%s" % (self.d // lepgcd, lepgcd), "s")

    def traitement(self, final=False):
        """**traitement**\ (*object*,\ *self*)

        Finit la mise au même dénominateur ou la simplification de la fraction.
        Si *final* est vrai, alors essaie de simplifier la fraction.

            >>> from pyromaths.classes.Fractions import Fraction
            >>> repr(Fraction("3*4", "3*7", "r").traitement())
            Fraction(12, 21)
            >>> repr(Fraction("3*4", "3*7", "s").traitement())
            Fraction(4, 7)
            >>> repr(Fraction(12, 21).traitement())
            Fraction(12, 21)
            >>> repr(Fraction(12, 21).traitement(True))
            Fraction("4*3", "7*3", "s")

        :param: final
        :type: boolean
        :rtype: Fraction
        """
        if self.code == "r":
            if isinstance(self.n, str): n = eval(self.n)
            else: n = self.n
            if isinstance(self.d, str): d = eval(self.d)
            else: d = self.d
            return Fraction(n, d)
        if self.code == "s":
            if isinstance(self.n, str): n = eval(self.n)
            else: n = self.n
            if isinstance(self.d, str): d = eval(self.d)
            else: d = self.d
            s = pgcd(n, d)
            return Fraction(n // s, d // s)
        if final:
            return self.decompose()
        # Pas de traitement spécifique nécessaire
        return self



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import math
from random import randrange

from .Conversions import degres, radians


def choix_points(n):
    """
    choisit n points parmi A, B, C, ..., Z
    @param n: nombre de points \xc3\xa0 choisir
    @type n: integer
    """
    points = [chr(i + 65) for i in range(26)]
    liste = []
    for i in range(n):
        liste.append(points.pop(randrange(len(points))))
    return liste


couples_pythagore = (
    (12, 16, 20),
    (15, 20, 25),
    (10, 24, 26),
    (20, 21, 29),
    (18, 24, 30),
    (16, 30, 34),
    (21, 28, 35),
    (12, 35, 37),
    (15, 36, 39),
    (24, 32, 40),
    (27, 36, 45),
    (14, 48, 50),
    (30, 40, 50),
    (24, 45, 51),
    (20, 48, 52),
    (28, 45, 53),
    (33, 44, 55),
    (40, 42, 58),
    (36, 48, 60),
    (11, 60, 61),
    (16, 63, 65),
    (25, 60, 65),
    (33, 56, 65),
    (39, 52, 65),
    (32, 60, 68),
    (42, 56, 70),
    (48, 55, 73),
    (24, 70, 74),
    (21, 72, 75),
    (45, 60, 75),
    (30, 72, 78),
    (48, 64, 80),
    (18, 80, 82),
    (13, 84, 85),
    (36, 77, 85),
    (40, 75, 85),
    (51, 68, 85),
    (60, 63, 87),
    (39, 80, 89),
    (54, 72, 90),
    (35, 84, 91),
    (57, 76, 95),
    (65, 72, 97),
    (28, 96, 100),
    (60, 80, 100),
    (20, 99, 101),
    (48, 90, 102),
    (40, 96, 104),
    (63, 84, 105),
    (56, 90, 106),
    (60, 91, 109),
    (66, 88, 110),
    (36, 105, 111),
    (15, 112, 113),
    (69, 92, 115),
    (80, 84, 116),
    (45, 108, 117),
    (56, 105, 119),
    (72, 96, 120),
    (22, 120, 122),
    (27, 120, 123),
    (35, 120, 125),
    (44, 117, 125),
    (75, 100, 125),
    (32, 126, 130),
    (50, 120, 130),
    (66, 112, 130),
    (78, 104, 130),
    (81, 108, 135),
    (64, 120, 136),
    (88, 105, 137),
    (84, 112, 140),
    (55, 132, 143),
    (17, 144, 145),
    (24, 143, 145),
    (87, 116, 145),
    (100, 105, 145),
    (96, 110, 146),
    (48, 140, 148),
    (51, 140, 149),
    (42, 144, 150),
    (90, 120, 150),
    (72, 135, 153),
    (93, 124, 155),
    (60, 144, 156),
    (85, 132, 157),
    (84, 135, 159),
    (96, 128, 160),
    (36, 160, 164),
    (99, 132, 165),
    (65, 156, 169),
    (119, 120, 169),
    (26, 168, 170),
    (72, 154, 170),
    (80, 150, 170),
    (102, 136, 170),
    (52, 165, 173),
    (120, 126, 174),
    (49, 168, 175),
    (105, 140, 175),
    (78, 160, 178),
    (108, 144, 180),
    (19, 180, 181),
    (70, 168, 182),
    (33, 180, 183),
    (57, 176, 185),
    (60, 175, 185),
    (104, 153, 185),
    (111, 148, 185),
    (88, 165, 187),
    (114, 152, 190),
    (95, 168, 193),
    (130, 144, 194),
    (48, 189, 195),
    (75, 180, 195),
    (99, 168, 195),
    (117, 156, 195),
    (28, 195, 197),
    (56, 192, 200),
    (120, 160, 200),
)


def trouve_couples_pythagore(max):
    ls = []
    for i in range(26):
        for j in range(i - 1):
            a = i
            b = j + 1
            for k in range(140):
                x = (2 * k + 1) * (a ** 2 - b ** 2)
                y = (2 * k + 1) * (2 * a * b)
                z = (2 * k + 1) * (a ** 2 + b ** 2)
                if z > max:
                    break
                ls.append(tuple(sorted([x, y, z])))
    ls.sort(key=operator.itemgetter(2))
    cpt = 1
    while cpt < len(ls):
        if ls[cpt] == ls[cpt - 1]:
            ls.pop(cpt)
        else:
            cpt += 1
    return tuple(ls)


# ----------------------------------------------------
# Cotation des longeurs sur une figure psTricks
# A, B sont les coordonnées de deux points
# cotation_h écrit au dessus du segment
# ----------------------------------------------------

def cotation(A, B, longueur, couleur="", unite="cm"):
    (xA, yA) = A
    (xB, yB) = B
    if couleur != "":
        couleur = "\\color{%s}" % couleur
    return u"\\pcline[linestyle=none](%.3f,%.3f)(%.3f,%.3f)  \\bput{:U}{%s\\unit[%s]{%s}}" % (
        xA, yA, xB, yB, couleur, longueur, unite)


def cotation_h(A, B, longueur, couleur="", unite="cm"):
    (xA, yA) = A
    (xB, yB) = B
    if couleur != "":
        couleur = "\\color{%s}" % couleur
    return u"\\pcline[linestyle=none](%.3f,%.3f)(%.3f,%.3f)  \\aput{:U}{%s\\unit[%s]{%s}}" % (
        xA, yA, xB, yB, couleur, longueur, unite)


# def trouve_couples_pythagore(valeurmax):
#    (liste, listecouples) = ([], [])
#    for a in range(valeurmax):
#        liste.append(a ** 2)
#    for c in range(valeurmax):
#        for b in range(int((c + 1) / 2 ** .5)):
#            if liste.count((c + 1) ** 2 - (b + 1) ** 2):
#                a = liste.index((c + 1) ** 2 - (b + 1) ** 2)
#                listeinter = [c + 1, b + 1, a]
#                listeinter.sort()
#                if listeinter[0] > 9:
#                    listecouples.append(tuple(listeinter))
#
#                if listecouples==[] :
#                   listecouples.append(tuple(listeinter))
#                else :
#                    ajout=1
#                    for i in xrange(len(listecouples)) :
#                        if not(listeinter[2]%listecouples[i][2]) and not(listeinter[1]%listecouples[i][1]) and not(listeinter[0]%listecouples[i][0]) :
#                            if listeinter[2]//listecouples[i][2]==listeinter[1]//listecouples[i][1]==listeinter[0]//listecouples[i][0] : ajout=0
#                    if ajout :
#                        listeinter=[c+1,b+1,a]
#                        listeinter.sort()
#                        listecouples.append(tuple(listeinter))
#
#    return tuple(listecouples)
class Metapost:

    def __init__(self):
        self.text = []
        self.num = 1

    def fin(self):
        self.text.append("end;")
        return self

    def triangle(
            self,
            A,
            B,
            C,
            a=0,
            b=0,
            c=0,
            alpha=0,
            beta=0,
            gamma=0,
            rotation=0,
            angledroit=0,
    ):
        """Construit un triangle en metapost quelles que soient les données.
        La base est le côté [AB] de longueur c.

        @param A, b, C : nom des trois sommets (obligatoire)
        @param a, b, c : longueurs des trois côtés opposés au sommet de même nom
        @param alpha, beta, gamma : mesure des 3 angles de sommets A, B et C
        @param rotation: mesure en degrés de l'angle de la rotation de centre A
                         appliquée au triangle
        @param angledroit: doit-on afficher l'angle droit ?
        """

        self.text.append("beginfig(%s);\n" % self.num)
        self.text.append("u:=1cm;\n")
        self.text.append("pair %s, %s, %s, m[];\n" % (A, B, C))
        self.text.append("picture $;\n")
        self.text.append("  %s=origin;\n" % A)
        if angledroit:
            marques = [0, 0, 0, "%s \degres" % alpha, "%s \degres" % beta, "%s \degres" % gamma]
            if a: marques[0] = "%s cm" % a
            if b: marques[1] = "%s cm" % b
            if c: marques[2] = "%s cm" % c
            points = [A, B, C]
        # on donne les trois longueurs des 3 côtés

        if a and b and c:
            alpha = degres(math.acos(b ** 2 + c ** 2 - a ** 2 / (2 * b * c)))
            beta = degres(math.acos(c ** 2 + a ** 2 - b ** 2 / (2 * c * a)))
        elif a and b and gamma:

            # Un angle et les deux côtés adjacents

            (c, beta, alpha) = self.triangle_angle_cotes_adjacents(a, b, gamma)
        elif b and c and alpha:
            (a, gamma, beta) = self.triangle_angle_cotes_adjacents(b, c, alpha)
        elif c and a and beta:
            (b, alpha, gamma) = self.triangle_angle_cotes_adjacents(c, a, beta)
        elif b and c and beta:

            # Un angle, le côté opposé et un côté adjacent

            (a, alpha, gamma) = self.triangle_angle_cote_adjacent_cote_oppose(b, c, beta)
        elif b and a and beta:
            (c, gamma, alpha) = self.triangle_angle_cote_adjacent_cote_oppose(b, a, beta)
        elif a and b and alpha:
            (c, gamma, beta) = self.triangle_angle_cote_adjacent_cote_oppose(a, b, alpha)
        elif a and c and alpha:
            (b, beta, gamma) = self.triangle_angle_cote_adjacent_cote_oppose(a, c, alpha)
        elif c and a and gamma:
            (b, beta, alpha) = self.triangle_angle_cote_adjacent_cote_oppose(c, a, gamma)
        elif c and b and gamma:
            (a, alpha, beta) = self.triangle_angle_cote_adjacent_cote_oppose(c, b, gamma)
        elif alpha and beta and c:

            # Deux angles et le côté commun

            pass  # on sait faire
        elif beta and gamma and a:
            c = a * math.sin(gamma * math.pi / 180) / math.sin((beta + gamma) * math.pi / 180)
            alpha = (180 - beta) - gamma
        elif alpha and gamma and b:
            c = b * math.sin(gamma * math.pi / 180) / math.sin((alpha + gamma) * math.pi / 180)
            beta = (180 - alpha) - gamma
        elif a and alpha and beta:
            # Deux angles et un côté non commun
            c = a * math.sin((alpha + beta) * math.pi / 180) / math.sin(alpha * math.pi / 180)
        elif a and alpha and gamma:
            c = a * math.sin(gamma * math.pi / 180) / math.sin(alpha * math.pi / 180)
            beta = (180 - alpha) - gamma
        elif b and beta and alpha:
            c = b * math.sin((alpha + beta) * math.pi / 180) / math.sin(beta * math.pi / 180)
        elif b and beta and gamma:
            c = b * math.sin(gamma * math.pi / 180) / math.sin(beta * math.pi / 180)
            alpha = (180 - beta) - gamma
        elif c and alpha and gamma:
            beta = (180 - alpha) - gamma
        elif c and beta and gamma:
            alpha = (180 - beta) - gamma
        alpha = alpha + rotation
        beta = beta - rotation

        self.text.append("  %s= (%s*u, 0) rotated %s;\n" % (B, c, rotation))
        self.text.append("  %s = %s + whatever*dir(%s);\n" % (C, A, alpha))
        self.text.append("  %s = %s + whatever*dir(180-%s);\n" % (C, B, beta))
        self.text.append("  draw %s -- %s -- %s -- cycle;\n" % (A, B, C))

        # Codage de l'angle droit s'il y en a un.

        if angledroit:
            if alpha - rotation == 90:
                self.text.append("  draw (%s+5*unitvector(%s-%s))--(%s+5*unitvector(%s-%s)+\n" % (A, B, A, A, B, A))
                self.text.append("      5*unitvector(%s-%s))--(%s+5*unitvector(%s-%s));\n" % (C, A, A, C, A))
            elif beta + rotation == 90:
                self.text.append("  draw (%s+5*unitvector(%s-%s))--(%s+5*unitvector(%s-%s)+\n" % (B, A, B, B, A, B))
                self.text.append("      5*unitvector(%s-%s))--(%s+5*unitvector(%s-%s));\n" % (C, B, B, C, B))
            elif gamma == 90:
                self.text.append("  draw (%s+5*unitvector(%s-%s))--(%s+5*unitvector(%s-%s)+\n" % (C, A, C, C, A, C))
                self.text.append("      5*unitvector(%s-%s))--(%s+5*unitvector(%s-%s));\n" % (B, C, C, B, C))
            for i in range(3):
                if marques[i]:
                    self.text.append(
                        "  m3:=unitvector(%s-%s) rotated 90;\n" % (points[(i + 1) % 3], points[(i + 2) % 3]))
                    self.text.append("  $:=image(\n")
                    self.text.append("    label(btex %s etex rotated angle(%s-%s),(%s+%s)/2+2mm*m3);\n" % (
                        marques[i], points[(i + 1) % 3], points[(i + 2) % 3], points[(i + 1) % 3], points[(i + 2) % 3]))
                    self.text.append("    );\n  draw $;\n")
        self.text.append("  label.llft(btex $%s$ etex, %s);\n" % (A, A))
        self.text.append("  label.lrt(btex $%s$ etex, %s);\n" % (B, B))
        self.text.append("  label.top(btex $%s$ etex, %s);\n" % (C, C))
        self.text.append("endfig;\n\n")
        self.num = self.num + 1
        return self

    def triangle_angle_cotes_adjacents(self, a, b, gamma):
        c = math.sqrt(a ** 2 + b ** 2 - 2 * a * b * math.cos(radians(gamma)))
        alpha = 90. - gamma / 2 + degres(math.atan((a - b) / (a + b)) / math.tan(radians(gamma / 2)))
        beta = 90. - gamma / 2 - degres(math.atan((a - b) / (a + b) / math.tan(radians(gamma / 2))))
        return (c, beta, alpha)

    def triangle_angle_cote_adjacent_cote_oppose(self, b, c, beta):
        if b <= c * math.sin(beta * math.pi / 180):
            alpha = gamma = a = 0  # Pas possible de résoudre
        else:
            gamma = degres(math.asin(c * math.sin(beta * math.pi / 180)) / b)
            alpha = (180 - beta) - gamma
            a = math.sqrt(b ** 2 - c ** 2 * math.sin(beta * math.pi / 180) ** 2) + c * math.cos(beta * math.pi / 180)
        return (a, alpha, gamma)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from functools import reduce
# from math import sqrt
from random import shuffle, randrange
from decimal import Decimal

from pyromaths import ex
from pyromaths.classes.Fractions import Fraction
from pyromaths.classes.SquareRoot import SquareRoot
from pyromaths.classes.PolynomesCollege import Polynome, factoriser
from pyromaths.outils import Priorites3
from pyromaths.outils.Arithmetique import carrerise, pgcd, valeur_alea

"""Exercice de seconde : Chapitre Second degré."""

import random

from pyromaths.ex import Jinja2Exercise
from pyromaths.outils.jinja2utils import facteur

def signe(nombre):
    """Renvoit une chaîne contenant le signe de l'argument."""
    if nombre < 0:
        return "-"
    return "+"
dummy=SquareRoot([1,1]) # Pour que l'IDE sache que l'import de SquareRoot est obligatoire

class BilanTrinomeSansDiscriminant(Jinja2Exercise):
    """Bilan sur les trinômes"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        while True:
            a = Decimal(random.choice([-1, 1]) * random.choice([0.5, 2]))
            x1 = Decimal(random.choice([-1, 1]) * random.randint(2, 15))
            x2 = Decimal(random.choice([-1, 1]) * random.randint(2, 15))

            b = -a * (x1 + x2)
            c = a * x1 * x2

            alpha = -b / (2*a)
            beta = a * (alpha**2) + b * alpha + c

            if alpha == 0 or beta == 0:
                continue
            if b == 0 or c == 0:
                continue
            if beta == c:
                continue

            break

        self.context = {
            "a": a,
            "b": b,
            "c": c,
            "x1": x1,
            "x2": x2,
            "alpha": alpha,
            "absalpha": abs(alpha), # Valeur absolue de alpha
            "signealpha": alpha / abs(alpha), # Signe de alpha (qui est non nul)
            "beta": beta,
            }

    @property
    def environment(self):
        environment = super().environment
        environment.filters.update({
            'facteur': facteur,
            'min': min,
            'max': max,
            'abs': abs,
            'signe': signe,
            })
        return environment

def creerPolydegre2(nb_racines=2, rac_radical=True, rac_quotient=False):
    if nb_racines == 2:
        redo = True
        while redo:
            a = randrange(1, 4) * (-1) ** randrange(2)
            alpha = randrange(1, 10) * (-1) ** randrange(2)
            beta = randrange(1, 10)
            gamma = [1, randrange(1, 6)][rac_radical]
            if rac_quotient:
                den = randrange(2, 6)
                while pgcd(alpha, den) != 1 or pgcd(beta, den) != 1:
                    den = randrange(2, 6)
                alpha = Fraction(alpha, den)
                beta = Fraction(beta, den)
            b = -2 * alpha * a
            c = a * (alpha ** 2 - gamma * beta ** 2)
            if abs(c) <= 10 and c != 0 and not factoriser(repr(Polynome([[a, 2], [b, 1], [c, 0]]))): redo = False
            if c.denominator != 1:
                c = 'Fraction(%s, %s)' % (c.numerator, c.denominator)
            else:
                c = c.numerator
            if b.denominator != 1:
                b = 'Fraction(%s, %s)' % (b.numerator, b.denominator)
            else:
                b = b.numerator
        return Polynome([[a, 2], [b, 1], [c, 0]])
    elif nb_racines == 1:
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        return Polynome([[a ** 2, 2], [2 * a * b, 1], [b ** 2, 0]])
    else:
        pol = [[valeur_alea(-9, 9), 2 - dummy] for dummy in range(3)]
        while pol[1][0] ** 2 - 4 * pol[0][0] * pol[2][0] >= 0:
            pol = [[valeur_alea(-9, 9), 2 - dummy] for dummy in range(3)]
        return Polynome(pol)

class Sd1FormeCanonique(ex.TexExercise):
    """Forme canonique"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        m = [[1, 2], [2 * randrange(1, 10) * (-1) ** randrange(2), 1], [randrange(1, 10) * (-1) ** randrange(2), 0]]
        pol = [[['Polynome(%s, "x", details=0)' % m]]]
        pol[0].extend(self.resolution(m))
        m = [[1, 2], [(2 * randrange(1, 6) + 1) * (-1) ** randrange(2), 1], [randrange(1, 10) * (-1) ** randrange(2), 0]]
        pol.append([['Polynome(%s, "x", details=0)' % m]])
        pol[1].extend(self.resolution(m))
        a, b = randrange(1, 10), randrange(1, 10)
        m = [[a ** 2, 2], [2 * a * b * (-1) ** randrange(2), 1], [b ** 2, 0]]
        pol.append([['Polynome(%s, "x", details=0)' % m]])
        if m[1][0] < 0: pol[2].extend(self.id_rem(a, b, '-'))
        else: pol[2].extend(self.id_rem(a, b, '+'))
        m = [[randrange(2, 6) * (-1) ** randrange(2), 2], [randrange(1, 10) * (-1) ** randrange(2), 1], [randrange(1, 10) * (-1) ** randrange(2), 0]]
        pol.append([['Polynome(%s, "x", details=0)' % m]])
        fracb = Fraction(m[1][0], m[0][0]).simplifie()
#        if fracb.d == 1: fracb = fracb.n
        fracc = Fraction(m[2][0], m[0][0]).simplifie()
#        if fracc.d == 1: fracc = fracc.n
        pol[3].append(['%s' % m[0][0], '*', '(', 'Polynome(%s, "x", details=0)' % [[1, 2], [fracb, 1], [fracc, 0]], ')'])
        pol[3].extend(self.resolution([[1, 2], [fracb, 1], [fracc, 0]], ['%s' % m[0][0], '*', '('], [')']))
        shuffle(pol)
        self.exercice = pol

    def resolution(self, m, pre=[], post=[]):
        sgn = '+-'[m[1][0] < 0]
        if isinstance(m[1][0], Fraction):
            b = Priorites3.priorites(abs(m[1][0]) / 2)[-1][0]
        elif m[1][0] % 2:
            b = 'Fraction(%s, 2)' % abs(m[1][0])
        else:
            b = abs(m[1][0]) // 2
        fc = ['Polynome("%sx%s%s")' % (m[0][0], sgn, b), '**', '2']
        reste = ['-']
        if m[2][0] > 0 or isinstance(m[2][0], Fraction):
            reste.extend(Priorites3.splitting('%s**2+%r' % (b, m[2][0])))
        else:
            reste.extend(Priorites3.splitting('%s**2%r' % (b, m[2][0])))
        etapes = list(pre)
        etapes.extend(fc)
        etapes.extend(reste)
        etapes.extend(post)
        etapes = [etapes]
        for unreste in Priorites3.priorites(''.join(reste)):
            calcul = list(pre)
            calcul.extend(fc)
            if unreste[0][0] != '-':
                calcul.append('+')
            else:
                calcul.append('-')
                unreste[0] = unreste[0][1:]
            calcul.extend(unreste)
            calcul.extend(post)
            etapes.append(calcul)
        if isinstance(b, str):  # On supprime deux étapes trop détaillée UGLY
            # TODO: Corriger la classe Fractions pour qu'elle gère plusieurs niveaux de détails.
            etapes.pop(1)
            etapes.pop(1)
        if pre:
            calcul = pre[0:-1]
            calcul.extend(fc)
            fc = list(calcul)
            reste = Priorites3.priorites(''.join(reste))[-1]
            reste.extend(['*', pre[0]])
            for unreste in Priorites3.priorites(''.join(reste)):
                calcul = list(fc)
                if unreste[0][0] != '-':
                    calcul.append('+')
                else:
                    calcul.append('-')
                    unreste[0] = unreste[0][1:]
                calcul.extend(unreste)
                etapes.append(calcul)
        return etapes

    def id_rem(self, a, b, sgn):
        etapes = [['Polynome("%sx%s%s")' % (a, sgn, b), '**', '2']]
        if a != 1:
            etapes.append(['(', '%s' % a, '*', 'Polynome("x%sFraction(%s, %s)")' % (sgn, b, a), ')', '**', '2' ])
            frac = Fraction(b, a).simplifie()
            etapes.append(['%s' % (a ** 2), '*', 'Polynome("x%s%r")' % (sgn, frac), '**', '2'])
        return etapes

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(_(u'Donner la forme canonique des polynômes $P$ , $Q$ , $R$ et $S$ .'))
        exo.append(r'\begin{align*}')
        noms = [r'P\,(x) &= ', r'Q\,(x) &= ', r'R\,(x) &= ', r'S\,(x) &= ']
        exercice = list(self.exercice)
        sol = ''
        for i in range(len(exercice)):
            exercice[i][0] = Priorites3.texify(exercice[i][0])
            sol += noms[i] + exercice[i][0][0]
            if i < len(exercice) - 1: sol += r' & '
            else: sol += r' \\ '
        exo.append(sol)
        exo.append(r'\end{align*}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(_(u'Donner la forme canonique des polynômes $P$ , $Q$ , $R$ et $S$ .'))
        noms = [r'P\,(x) &= ', r'Q\,(x) &= ', r'R\,(x) &= ', r'S\,(x) &= ']
        exercice = list(self.exercice)
        for i in range(len(exercice)):
            exercice[i] = Priorites3.texify(exercice[i])
            exercice[i][0] = noms[i] + exercice[i][0]  # [0]
            for j in range(1, len(exercice[i]) - 1):
                exercice[i][j] = r' &= ' + exercice[i][j]
            exercice[i][-1] = r'\Aboxed{' + noms[i] + exercice[i][-1] + r'}'
        tri = []
        for i in range(len(exercice)):
            tri.append((i, len(exercice[i])))
        tri = sorted(tri, key=lambda nblgn: nblgn[1])
        if tri[0][0] < tri[1][0]:
            exercice[tri[0][0]].extend(exercice.pop(tri[1][0]))
        else:
            exercice[tri[1][0]].extend(exercice.pop(tri[0][0]))
        exo.append(r'\begin{align*}')
        for j in range(max(len(exercice[0]), len(exercice[1]), len(exercice[2]))):
            sol = ''
            for i in range(3):
                if j < len(exercice[i]):
                    sol += exercice[i][j]
                else:
                    sol += r'& '
                if i == 2:
                    sol += r'\\'
                else:
                    sol += r' & '
            exo.append(sol)
        exo.append(r'\end{align*}')
        return "\n".join(exo)

class Sd2aRacines(ex.TexExercise):
    #TODO: ./utils/pyromaths generate Sd2aRacines:1, 2 et 3 ne fonctionnent pas
    """Racines d'un polynôme de degré 2"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        pol = [creerPolydegre2(nb_racines=2, rac_radical=True, rac_quotient=False)]
        pol.append(creerPolydegre2(nb_racines=1))
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a ** 2, 2], [-b ** 2, 0]])
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        while a * b == -64:
            # sqrt{8} est trop long à décomposer en une demi-ligne
            a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a, 2], [b, 0]])
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a, 2], [b, 1]])
        pol.pop(randrange(1, len(pol)))
        pol.pop(randrange(1, len(pol)))
        shuffle(pol)
        for i in range(3):
            m = list(pol[i])
            shuffle(m)
            pol[i] = m
            #print(str(Polynome(pol[i], "x")))
        self.exercice = pol

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(_(u'Déterminer les racines des polynômes :'))
        exo.append('\\begin{align*}')
        noms = [r'P\,(x) &= ', r'Q\,(x) &= ', r'R\,(x) &= ']
        r = ''
        for i in range(3):
            r += noms[i] + str(Polynome(self.exercice[i], 'x'))
            if i < 2: r += ' & '
        exo.append(r)
        exo.append('\\end{align*}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(_(u'Déterminer les racines des polynômes :\\par'))
        noms = [r'P\,(x) &= ', r'Q\,(x) &= ', r'R\,(x) &= ']
        r = ''
        question = [[], [], []]
        for i in range(3):
            p = []
            m = Polynome(list(self.exercice[i])).ordonne()
            if factoriser('%r' % Polynome(m)):
                p = [factoriser('%r' % Polynome(m))]
                while factoriser(p[-1]):
                    p.append(factoriser(p[-1]))
            if p and eval(Priorites3.splitting(p[-1])[0]).degre() > 0:
                tmp = Priorites3.texify([Priorites3.splitting(p[j]) for j in range(len(p))])
                question[i].append('{$\\! \\begin{aligned}')
                question[i].append(noms[i] + str(Polynome(m, 'x')) + r'\\')
                question[i].append('\\\\\n'.join(['&=%s' % (tmp[j]) for j in range(len(tmp))]))
                question[i].append(r'\end{aligned}$}\par')
                lp = Priorites3.splitting(p[-1])
                racines = []
                for e in lp:
                    if e[:9] == 'Polynome(':
                        e = eval(e)
                        if len(e) == 2:
                            racines.append(str(Fraction(-e[1][0], e[0][0]).simplifie()))
                        else:
                            racines.append('0')
                if len(racines) > 1:
                    question[i].append(_(u'\\underline{Les racines de $%s$ sont }\\fbox{$%s$}') % (noms[i].rstrip(r' &= '), '$}\\underline{ et }\\fbox{$'.join(racines)))
                elif len(racines) == 1:
                    question[i].append(_(u'\\underline{L\'unique racine de $%s$ est }\\fbox{$%s$}') % (noms[i].rstrip(r' &= '), racines[0]))
            elif len(m) == 2 and m[0][1] == 2 and m[1][1] == 0 and m[0][0] * m[1][0] > 0:
                question[i].append('$' + noms[i] + str(Polynome(m, 'x')) + r'$\par')
                question[i][-1] = question[i][-1].replace('&', '')
                if m[1][0] > 0: question[i].append('$' + noms[i][:7] + ' \\geqslant %r$' % m[1][0])
                else: question[i].append('$' + noms[i][:7] + ' \\leqslant %r$' % m[1][0])
                question[i].append(_(u'car un carré est toujours positif.\\par\n\\underline{$%s$ n\'a donc pas de racine.}') % (noms[i].rstrip(r' &= ')))
            else:
                question[i].append('$' + noms[i] + str(Polynome(m, 'x')) + r'\quad$')
                question[i][-1] = question[i][-1].replace('&', '')
                question[i].append(_(u'On calcule le discriminant de $%s$ avec $a=%s$, $b=%s$ et $c=%s$ :\\par\\medskip') % (noms[i].rstrip(r' &= '), m[0][0], m[1][0], m[2][0]))
                question[i].append(r'\begin{tabularx}{\linewidth}[t]{XXX}')
                question[i].append(r'{$\! \begin{aligned}')
                if m[1][0]>0:
                    sol = [[str(m[1][0]), '**', '2', '-', '4', '*', str(m[0][0]), '*', str(m[2][0])]]
                    sol.extend(Priorites3.priorites('%s**2-4*%s*%s' % (m[1][0], m[0][0], m[2][0])))
                else:
                    sol = [['(', str(m[1][0]), ')', '**', '2', '-', '4', '*', str(m[0][0]), '*', str(m[2][0])]]
                    sol.extend(Priorites3.priorites('(%s)**2-4*%s*%s' % (m[1][0], m[0][0], m[2][0])))
                solTeX = Priorites3.texify(sol)
                for s in solTeX:
                    question[i].append(u'\\Delta &= %s\\\\' % s)
                question[i].append(r'\end{aligned}$}')
                question[i].append(r'&')
                question[i].append(r'{$\! \begin{aligned}')
                delta = sol[-1][0]
                #print(sol)
                sol = [['Fraction(SquareRoot([[%s, None], [-1, %s]]),\'2*%s\')' % (-m[1][0], delta, m[0][0])]]
                sol.extend(Priorites3.priorites(sol[0][0]))
                sol = Priorites3.texify(sol)
                for s in sol:
                    question[i].append(u'x_1 &= %s\\\\' % s)
                racines = [sol[-1]]
                question[i].append(r'\end{aligned}$}')
                question[i].append(r'&')
                question[i].append(r'{$\! \begin{aligned}')
                sol = [['Fraction(SquareRoot([[%s, None], [1, %s]]),\'2*%s\')' % (-m[1][0], delta, m[0][0])]]
                sol.extend(Priorites3.priorites(sol[0][0]))
                sol = Priorites3.texify(sol)
                for s in sol:
                    question[i].append(u'x_2 &= %s\\\\' % s)
                racines.append(sol[-1])
                question[i].append(r'\end{aligned}$}')
                question[i].append(r'\end{tabularx}\par')
                question[i].append(_(u'\\underline{Les racines de $%s$ sont }\\fbox{$%s$}') % (noms[i].rstrip(r' &= '), _('$}\\underline{ et }\\fbox{$').join(racines)))
                if i == 1: question.append(question[1])
        if len(question) == 4:
            question.pop(1)
        if question[0][0][-6:] == r'\quad$':
            question[1].insert(0, r'\par\medskip\begin{tabularx}{\linewidth}[t]{XX}')
            question[2].insert(0, r'&')
            question[2].append(r'\end{tabularx}\par\medskip')
        else:
            question[0].insert(0, r'\begin{tabularx}{\linewidth}[t]{XX}')
            question[1].insert(0, r'&')
            question[1].append(r'\end{tabularx}\par\medskip')
        for i in range(3): exo.extend(question[i])
        return "\n".join(exo)

class _Sd2bEquations(ex.TexExercise):
    """Équations et polynômes de degré 2"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        pol = [creerPolydegre2(nb_racines=2, rac_radical=False, rac_quotient=False)]
        pol.append(creerPolydegre2(nb_racines=1))
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a ** 2, 2], [-b ** 2, 0]])
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a, 2], [b, 1]])
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a, 2], [b, 0]])
        pol.pop(randrange(1, len(pol)))
        pol.pop(randrange(1, len(pol)))
        pol.pop(randrange(1, len(pol)))
        shuffle(pol)
        exercice = [pol]
        lval = [[[randrange(2, 10) * (-1) ** randrange(2), 1], [randrange(2, 10) * (-1) ** randrange(2), 0]] for dummy in range(3)]
        a, b, c = -lval[2][0][0] * lval[1][0][0], lval[0][0][0] - lval[2][0][0] * lval[1][1][0] - lval[2][1][0] * lval[1][0][0], lval[0][1][0] - lval[2][1][0] * lval[1][1][0]
        delta = b ** 2 - 4 * a * c
        while delta < 0 or carrerise(delta) != 1:
            lval = [[[randrange(2, 10) * (-1) ** randrange(2), 1], [randrange(2, 10) * (-1) ** randrange(2), 0]] for dummy in range(3)]
            a, b, c = -lval[2][0][0] * lval[1][0][0], lval[0][0][0] - lval[2][0][0] * lval[1][1][0] - lval[2][1][0] * lval[1][0][0], lval[0][1][0] - lval[2][1][0] * lval[1][1][0]
            delta = b ** 2 - 4 * a * c
        # print delta, Polynome([[a, 2], [b, 1], [c, 0]]), '\cfrac{%s}{%s}' % (-b - sqrt(delta), 2 * a), '\cfrac{%s}{%s}' % (-b + sqrt(delta), 2 * a)
        exercice.append(lval)
        shuffle(exercice)
        self.exercice = exercice

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(_(u'Résoudre les équations :'))
        exo.append('\\begin{align*}')
        for e in self.exercice:
            if len(e) == 2:
                exo.append(Priorites3.texify([[repr(Polynome(e[0])), '*', repr(Polynome(e[1]))]])[0] + ' &= 0 & ')
            else:
                exo.append(r'\cfrac{%s}{%s} &= %s & ' % (Polynome(e[0]), Polynome(e[1]), Polynome(e[2])))
        exo[-1] = exo[-1][:-3]  # Suppression du dernier  " &"
        exo.append('\\end{align*}')
        return "\n".join(exo)

class _Sd3aSigne(ex.TexExercise):
    """Signe d'un polynôme de degré 2"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        pol = [[valeur_alea(-9, 9), 2 - dummy] for dummy in range(3)]
        while pol[1][0] ** 2 - 4 * pol[0][0] * pol[2][0] >= 0:
            pol = [[valeur_alea(-9, 9), 2 - dummy] for dummy in range(3)]
        exercice = [list(pol)]

        val = [valeur_alea(-9, 9), valeur_alea(-9, 9)]
        val.append(Fraction(valeur_alea(-9, 9), val[0]))
        while val[2].d == 1:
            val = [valeur_alea(-9, 9), valeur_alea(-9, 9)]
            val.append(Fraction(valeur_alea(-9, 9), val[0]))
        sgn = -val[0] // abs(val[0])
        pol = [[val[0], 2], [(-val[0] * (val[1] * val[2].d + val[2].n)) // val[2].d, 1], [(val[0] * val[1] * val[2].n) // val[2].d, 0]]
        shuffle(pol)
        exercice.append([pol, val[1], val[2]])

        val = [sgn * valeur_alea(-9, 9), valeur_alea(-9, 9)]
        val.append(Fraction(valeur_alea(-9, 9), val[0]).simplifie())
        while isinstance(val[2], int) or val[2].d == 1:
            val = [sgn * valeur_alea(-9, 9), valeur_alea(-9, 9)]
            val.append(Fraction(valeur_alea(-9, 9), val[0]))
        pol = [[val[0], 2], [(-val[0] * (val[1] * val[2].d + val[2].n)) // val[2].d, 1], [(val[0] * val[1] * val[2].n) // val[2].d, 0]]
        shuffle(pol)
        exercice.append([pol, val[1], val[2]])

        self.exercice = exercice

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Déterminer le signe du polynôme $P\\,(x) = %s$') % Polynome(self.exercice[0]))
        exo.append(_(u'\\item Le polynôme $Q\\,(x) = %s$ admet deux racines $%s$ et $%s\\,$. Dresser son tableau de signes.') \
            % (Polynome(self.exercice[1][0]), self.exercice[1][1], self.exercice[1][2]))
        exo.append(_(u'\\item Le polynôme $R\\,(x) = %s$ admet deux racines $%s$ et $%s\\,$. Dresser son tableau de signes.') \
            % (Polynome(self.exercice[2][0]), self.exercice[2][1], self.exercice[2][2]))
        exo.append('\\end{enumerate}')
        return "\n".join(exo)

class _Sd3bInequations(ex.TexExercise):
    """Inéquations et polynômes de degré 2"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        pol = creerPolydegre2(nb_racines=2, rac_radical=False, rac_quotient=False).monomes
        pol2 = [[valeur_alea(-9, 9), 1], [valeur_alea(-9, 9), 0]]
        shuffle(pol)
        shuffle(pol2)
        p = [pol, pol2]
        shuffle(p)
        p.append(['<', '>', '\\leqslant{}', '\\geqslant{}'][randrange(4)])
        self.exercice = p

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(_(u'Résoudre l\'inéquation : $\qquad \\cfrac{%s}{%s} %s 0$') % (Polynome(self.exercice[0]),
                                                                                  Polynome(self.exercice[1]),
                                                                                  self.exercice[2]))
        return "\n".join(exo)

class _Sd4Factorisation(ex.TexExercise):
    """Racines et factorisation d'un polynôme de degré 2"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        val = [valeur_alea(-9, 9), valeur_alea(-9, 9)]
        val.append(Fraction(valeur_alea(-9, 9), val[0]))
        while val[2].d == 1:
            val = [valeur_alea(-9, 9), valeur_alea(-9, 9)]
            val.append(Fraction(valeur_alea(-9, 9), val[0]))

        pol = [[val[0], 2], [(-val[0] * (val[1] * val[2].d + val[2].n)) // val[2].d, 1], [(val[0] * val[1] * val[2].n) // val[2].d, 0]]
        shuffle(pol)
        exercice = [[list(pol), val[1], val[2]]]

        pol = [creerPolydegre2(nb_racines=0).monomes]
        pol.append(creerPolydegre2(nb_racines=1).monomes)
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        sgn = [1, -1][randrange(2)]
        pol.append([[sgn * a ** 2, 2], [-sgn * b ** 2, 0]])
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a, 2], [b, 1]])
        a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        while abs(pgcd(a, b)) != 1:
            a, b = valeur_alea(-9, 9), valeur_alea(-9, 9)
        pol.append([[a, 2], [b, 0]])
        pol.pop(randrange(1, len(pol)))
        pol.pop(randrange(1, len(pol)))
        pol.pop(randrange(1, len(pol)))
        shuffle(pol)
        exercice.append(pol)

        self.exercice = exercice

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Le polynôme $\\quad P\\,(x) = %s \\quad$ admet deux racines $%s$ et $%s\\,$. Donner sa forme factorisée.\n') \
            % (Polynome(self.exercice[0][0]), self.exercice[0][1], self.exercice[0][2]))
        exo.append(_(u'\\item Factoriser si possible les polynômes $\quad Q\\,(x) = %s\\quad$ et $\\quad R\\,(x) = %s$.\n') % (Polynome(self.exercice[1][0]), Polynome(self.exercice[1][1])))
        exo.append(r'\end{enumerate}')

        return "\n".join(exo)

class _Sd5Caracteristiques(ex.TexExercise):
    """Caractéristiques d'une parabole"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        val = [valeur_alea(-9, 9), valeur_alea(-9, 9) , valeur_alea(-9, 9)]
        pol = Polynome([[val[0], 2], [(-val[0] * (val[1] + val[2])), 1], [(val[0] * val[1] * val[2]), 0]])
        while val[2] == val[1] or abs(val[0] * val[1] * val[2]) > 10 or abs(eval(pol((val[1] + val[2]) / 2))) > 10:
            val = [valeur_alea(-9, 9), valeur_alea(-9, 9) , valeur_alea(-9, 9)]
            pol = Polynome([[val[0], 2], [(-val[0] * (val[1] + val[2])), 1], [(val[0] * val[1] * val[2]), 0]])
        val = [[val[0], 2], [(-val[0] * (val[1] + val[2])), 1], [(val[0] * val[1] * val[2]), 0]]
        shuffle(val)
        lp = [Polynome(val)]

        val = [[valeur_alea(-9, 9), 2 - dummy] for dummy in range(3)]
        val[1][0] = valeur_alea(-9, 9) * val[0][0]
        pol = Polynome(val)
        while val[1][0] ** 2 - 4 * val[0][0] * val[2][0] >= 0 or abs(eval(pol(-val[1][0] / 2. / val[0][0]))) > 10:
            val = [[valeur_alea(-9, 9), 2 - dummy] for dummy in range(3)]
            val[1][0] = valeur_alea(-9, 9) * val[0][0]
            pol = Polynome(val)
        shuffle(val)
        pol = Polynome(val)
        lp.append(pol)
        shuffle(lp)

        self.exercice = lp

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(_(u'On donne les polynômes $\\quad p\\,(x) = %s \\quad$ et $\\quad Q\\,(x) = %s$.') % (self.exercice[0], self.exercice[1]))
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Donner les caractéristiques de leurs courbes respectives (sommet, intersections avec les axes du repère).'))
        exo.append(_(u'\\item Tracer l’allure de ces deux courbes sur un même graphique.'))
        exo.append(r'\end{enumerate}')

        return "\n".join(exo)

class _Sd6Parametre(ex.TexExercise):
    """Polynôme paramétré de degré 2"""

    tags = ["1re Spé math"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        [a, b, c, d] = [randrange(-5, 6) for dummy in range(4)]
        while a == 0 or c == 0 or a ** 2 * d - a * b * c + c ** 2 < 0 or carrerise(a ** 2 * d - a * b * c + c ** 2) != 1:
            [a, b, c, d] = [randrange(-5, 6) for dummy in range(4)]
        p1 = str(Polynome([[a, 1], [b, 0]], "m"))
        p2 = str(Polynome([[c, 1], [d, 0]], "m"))
        pol = [Polynome([[1, 2], [p1, 1], [p2, 0]]), randrange(3)]
        exercice = [list(pol)]

        v = [randrange(-4, 5) for dummy in range(6)]
        while v[0] == 0 or v[2] == v[4] == 0 or reduce(lambda x, y: x * y, v) != 0 or v[2] == v[3] == 0 or v[4] == v[5] == 0:
            v = [randrange(-4, 5) for dummy in range(6)]
        lp = [str(Polynome([[v[2 * i] // pgcd(v[2 * i], v[2 * i + 1]), 1], [v[2 * i + 1] // pgcd(v[2 * i], v[2 * i + 1]), 0]], "a")) for i in range(3)]
        pol = Polynome([[lp[0], 2], [lp[1], 1], [lp[2], 0]])
        vi = Fraction(-v[1], v[0])
        racine = randrange(-4, 5)
        while racine == vi or racine == 0:
            racine = randrange(-4, 5)
        if vi.d == 1 : vi = str(vi.n)
        else: vi = str(vi)
        exercice.append([list(pol), vi, racine])
        self.exercice = exercice

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item On donne le polynôme $\\quad P\\,(x) = %s\\quad$ où $m$ est un réel.\\par') % self.exercice[0][0])
        # TODO: Affichage des paramètres et parenthèses
        exo.append(_(u'Quelles sont les valeurs de $m$ pour lesquelles $P$ %s ?\n') % [_('a une seule racine'), _('n\'a pas de racine'),
                _('a deux racines distinctes')][self.exercice[0][1]])
        # exo.append(u'\\par Solution : Polynôme en m : $%s$\\par\n' % (Polynome([[a ** 2, 2], [2 * a * b - 4 * c, 1], [b ** 2 - 4 * d, 0]], "m"))
        # exo.append( u'Solution : discriminant $\\Delta_m = %s$\\par\n' % (16 * (a ** 2 * d - a * b * c + c ** 2))
        exo.append(_(u'\\item Soit $a$ un réel différent de $%s$. On donne $Q\\,(x) = %s$.\\par\n') % (self.exercice[1][1], Polynome(self.exercice[1][0])))
        exo.append(_(u'Déterminer $a$ pour que $%s$ soit une racine de $Q$.\n') % self.exercice[1][2])
        exo.append(r'\end{enumerate}')

        return "\n".join(exo)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils import Arithmetique
from pyromaths.outils.Affichage import TeX, tex_coef
from pyromaths.ex import LegacyExercise

# #
##------------------- MÉTHODE PAR COMBINAISON -------------------


def choix_valeurs(m):  # crée les valeurs du systeme de la forme a1.x+b1.y=c1 ; a2.x+b2.y=c2 et renvoie ((a1, b1, c1), (a2, b2, c2), (x, y))
    while True:
        c = [Arithmetique.valeur_alea(-m, m) for dummy in range(6)]
        if c[0] * c[3] - c[1] * c[2] and abs(c[0]) - 1 and abs(c[1]) - 1 and \
            abs(c[2]) - 1 and abs(c[3]) - 1 and abs(c[1] * c[2] - c[0] *
                c[3]) - 1 and abs(c[0]) - abs(c[2]) and abs(c[1]) - abs(c[3]) and \
            c[4] and c[5]:
            break

            # supprime des cas particuliers

    return ((c[0], c[1], c[0] * c[4] + c[1] * c[5]), (c[2], c[3], c[2] *
            c[4] + c[3] * c[5]), (c[4], c[5]))


def signe(a):  # retourne '+' si a est positif, '-' si a est négatif, '' si a est nul
    if a > 0:
        return '+'
    elif a < 0:
        return '-'
    else:
        return ''


def tex_systeme(v, p=None):  # renvoie l'écriture au format tex d'un système d'équations. v[] est de la forme ((a0, b0, c0), (a1, b1, c1), (x,y))
    if p == None:
        tv = (tex_coef(v[0][0], 'x'), signe(v[0][1]),
              tex_coef(abs(v[0][1]), 'y'), v[0][2],
              tex_coef(v[1][0], 'x'), signe(v[1][1]),
              tex_coef(abs(v[1][1]), 'y'), v[1][2])
        return '''\\left\\lbrace
\\begin{array}{rcrcl}
%s & %s & %s & = & %s \\\\\n      %s & %s & %s & = & %s
\\end{array}
\\right.''' % \
            tv
    else:
        tv = (
            tex_coef(v[0][0], 'x'),
            signe(v[0][1]),
            tex_coef(abs(v[0][1]), 'y'),
            v[0][2],
            '\\qquad\\hbox{\\footnotesize$\\mathit{(\\times %s)}$}' %
                tex_coef(p[0], '', bpn=1),
            tex_coef(v[1][0], 'x'),
            signe(v[1][1]),
            tex_coef(abs(v[1][1]), 'y'),
            v[1][2],
            '\\qquad\\hbox{\\footnotesize$\\mathit{(\\times %s)}$}' %
                tex_coef(p[1], '', bpn=1),
            )
        return '''\\left\\lbrace
\\begin{array}{rcrcll}
%s & %s & %s & = & %s & %s \\\\\n      %s & %s & %s & = & %s & %s
\\end{array}
\\right.''' % \
            tv


def combinaison1(v, a0, a1):
    return ((v[0][0] * a0, v[0][1] * a0, v[0][2] * a0), (v[1][0] * a1, v[1][1] *
            a1, v[1][2] * a1))


def combinaison2(v):  # renvoie les coefficients pour la somme des deux lignes
    return (v[0][0] + v[1][0], v[0][1] + v[1][1], v[0][2] + v[1][2])


def tex_comb2(v, c):
    if c[0]:
        tv = (tex_coef(v[0][0], 'x'), '\\cancel{' +
              tex_coef(v[0][1], 'y', bplus=1) + '}',
              tex_coef(v[1][0], 'x', bplus=1),
              '\\cancel{' + tex_coef(v[1][1], 'y', bplus=
              1) + '}', tex_coef(v[0][2], ''),
              tex_coef(v[1][2], '', bplus=1))
    else:
        tv = ('\\cancel{' + tex_coef(v[0][0], 'x') + '}',
              tex_coef(v[0][1], 'y', bplus=1),
              '\\cancel{' + tex_coef(v[1][0], 'x', bplus=
              1) + '}', tex_coef(v[1][1], 'y', bplus=1),
              tex_coef(v[0][2], ''), tex_coef(v[1][2],
              '', bplus=1))
    return '%s%s%s%s=%s%s' % tv


def tex_comb3(v):
    if v[0]:
        tv = (tex_coef(v[0], 'x'), tex_coef(v[2],
              ''))
    else:
        tv = (tex_coef(v[1], 'y'), tex_coef(v[2],
              ''))
    return '%s=%s' % tv


def tex_comb4(v):
    if v[0]:
        tv = (tex_coef(1, 'x'), v[2], v[0],
              tex_coef(v[2] // v[0], ''))
    else:
        tv = (tex_coef(1, 'y'), v[2], v[1],
              tex_coef(v[2] // v[1], ''))
    return '%s=\\frac{%s}{%s}=%s' % tv


def tex_equation(v, c):
    tv = (tex_coef(v[0][0], 'x'), tex_coef(v[0][1],
          'y', bplus=1), v[0][2])
    t = '$%s%s=%s\\quad\\text{et}\\quad ' % tv
    if c[0]:
        t = t + 'x=%s\\quad\\text{donc :}$\n' % v[2][0]
        tv = (tex_coef(v[0][0], ''), tex_coef(v[2][0],
              '', bpn=1), tex_coef(v[0][1], 'y', bplus=1),
              v[0][2])
        t = t + '\\[%s\\times %s %s=%s\\]\n' % tv
    else:
        t = t + 'y=%s\\quad\\text{donc :}$\n' % v[2][1]
        tv = (tex_coef(v[0][0], 'x'), tex_coef(v[0][1],
              '', bplus=1), tex_coef(v[2][1], '', bpn=1),
              v[0][2])
        t = t + '\\[%s %s\\times %s=%s\\]\n' % tv
    return t


def tex_eq2(v, c):
    if c[0]:
        tv = (tex_coef(v[0][1], 'y'), tex_coef(v[0][2],
              ''), tex_coef(-v[0][0] * v[2][0], '', bplus=
              1))
        t = '%s=%s%s' % tv
    else:
        tv = (tex_coef(v[0][0], 'x'), tex_coef(v[0][2],
              ''), tex_coef(-v[0][1] * v[2][1], '', bplus=
              1))
        t = '%s=%s%s' % tv
    return t


def tex_eq3(v, c):
    if c[0]:
        tv = (tex_coef(1, 'y'), tex_coef(v[0][2] -
              v[0][0] * v[2][0], ''), tex_coef(v[0][1],
              ''), tex_coef(v[2][1], ''))
        t = '%s=\\frac{%s}{%s}=%s' % tv
    else:
        tv = (tex_coef(1, 'x'), tex_coef(v[0][2] -
              v[0][1] * v[2][1], ''), tex_coef(v[0][0],
              ''), tex_coef(v[2][0], ''))
        t = '%s=\\frac{%s}{%s}=%s' % tv
    return t


def tex_verification(v):  # renvoie la vérification de lasolution du système d'équations.
    tv = (
        tex_coef(v[0][0], '\\times %s' % tex_coef(v[2][0],
                                '', bpn=1)),
        tex_coef(v[0][1], '\\times %s' % tex_coef(v[2][1],
                                '', bpn=1), bplus=1),
        tex_coef(v[0][0] * v[2][0], ''),
        tex_coef(v[0][1] * v[2][1], '', bplus=1),
        v[0][2],
        tex_coef(v[1][0], '\\times %s' % tex_coef(v[2][0],
                                '', bpn=1)),
        tex_coef(v[1][1], '\\times %s' % tex_coef(v[2][1],
                                '', bpn=1), bplus=1),
        tex_coef(v[1][0] * v[2][0], ''),
        tex_coef(v[1][1] * v[2][1], '', bplus=1),
        v[1][2],
        )
    return '''\\left\\lbrace
\\begin{array}{l}
%s %s=%s %s=%s \\\\\n      %s %s=%s %s=%s
\\end{array}
\\right.''' % \
        tv


def systemes(exo, cor, v):
    a = Arithmetique.ppcm(v[0][0], v[1][0])
    b = Arithmetique.ppcm(v[0][1], v[1][1])
    (a0, a1, b0, b1) = (a // v[0][0], -a // v[1][0], b // v[0][1], -b // v[1][1])
    if a0 < 0:
        (a0, a1) = (-a0, -a1)
    if b0 < 0:
        (b0, b1) = (-b0, -b1)
    if min(abs(a0), abs(a1)) > min(abs(b0), abs(b1)):
        (a0, a1) = (b0, b1)
    exo.append('$%s$' % tex_systeme(v))
    cor.append('$%s$' % tex_systeme(v, (a0, a1)))

    c1 = combinaison1(v, a0, a1)
    cor.append('''\\vspace{2ex}
  \\begin{multicols}{2}\\noindent
''')
    cor.append(u'\\[ ' + tex_systeme(c1) +
                     '\\quad\\text{\\footnotesize On ajoute les deux lignes}' + '\\] ')
    c2 = combinaison2(c1)
    cor.append(u'\\[ ' + tex_comb2(c1, c2) + '\\] ')
    cor.append(u'\\[ ' + tex_comb3(c2) + '\\] ')
    cor.append(u'\\[ \\boxed{' + tex_comb4(c2) + '} \\] ')
    cor.append('\\columnbreak\\par')
    cor.append(tex_equation(v, c2))
    cor.append(u'\\[ ' + tex_eq2(v, c2) + '\\] ')
    cor.append(u'\\[ \\boxed{' + tex_eq3(v, c2) + '} \\] ')
    cor.append('\\end{multicols}')
    cor.append(u"\\underline{La solution de ce système d'équations est $(x;~y)=(%s;~%s)$.}\\par" %
             v[2])
    cor.append(u'{Vérification : $' + tex_verification(v) + '$}')

def _tex_systemes():
    valeurs = choix_valeurs(10)
    exo = ['\\exercice', u"Résoudre le système d'équations suivant :"]
    cor = ['\\exercice*', u"Résoudre le système d'équations suivant :"]
    systemes(exo, cor, valeurs)
    return (exo, cor)

class tex_systemes(LegacyExercise):
    """Système d\'équations"""

    tags = ["Troisième"]
    function = _tex_systemes



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils.Arithmetique import ppcm, pgcd, signe, valeur_alea
from random import randrange
from pyromaths.ex import LegacyExercise

#
# ------------------- FRACTIONS -------------------


def den_com0(a, b):  # renvoie un tuple contenant les 2 nombres par lesquels multiplier les deux denominateurs pour obtenir leur ppcm
    c = ppcm(a[1], b[1])
    return (abs(c // a[1]), abs(c // b[1]))


def den_com1(a, b):  # renvoie un tuple contenant les fractions a et b avec le meme denominateur
    c = den_com0(a, b)
    sgn1 = signe(a[1])
    sgn2 = signe(b[1])
    return (((a[0] * c[0]) * sgn1, (a[1] * c[0]) * sgn1), ((b[0] * c[1]) * 
            sgn2, (b[1] * c[1]) * sgn2))


def somme(a, b, sgn):  # renvoie un tuple contenant la somme des fractions a et b ayant pour denominateur le ppcm de leurs denominateurs
    c = den_com1(a, b)
    if sgn == '+':
        return (c[0][0] + c[1][0], c[0][1])
    else:
        return (c[0][0] - c[1][0], c[0][1])


def simplifie(a):  # renvoie la fraction a simplifiee
    b = pgcd(a[0], a[1])
    if b != 1:
        return (a[0] // b, a[1] // b)
    else:
        return ''


def decomp_prod(a, b):  # renvoie un tuple contenant les deux fractions apres simplification et un tuple contenant les nb par lesquels on

    # simplifie le produit de fractions

    c = pgcd(a[0], b[1])
    d = pgcd(a[1], b[0])
    sgn1 = signe(a[1])
    sgn2 = signe(b[1])
    if c == d == 1:
        return (((sgn1 * a[0]) // c, (sgn1 * a[1]) // d), ((sgn2 * b[0]) // 
                d, (sgn2 * b[1]) // c), '')
    else:
        return (((sgn1 * a[0]) // c, (sgn1 * a[1]) // d), ((sgn2 * b[0]) // 
                d, (sgn2 * b[1]) // c), (c, d))


def produit(a, b):  # renvoie un tuple contenant le produit des fractions a et b
    sgn1 = signe(a[1] * b[1])
    return ((sgn1 * a[0]) * b[0], (sgn1 * a[1]) * b[1])


def inverse(a):  # renvoie l'inverse de la fraction a
    sgn1 = signe(a[0])
    return (sgn1 * a[1], sgn1 * a[0])


def tex_frac(a):  # renvoie l'ecriture au format tex de la fraction a
    if not isinstance(a, tuple):
        return ''
    else:
        a = ((a[0] * a[1]) // abs(a[1]), abs(a[1]))
        if a[1] == 1:
            if abs(a[1]) >= 1000:
                return '\nombre{%i}' % a[0]
            else:
                return '%i' % a[0]
        else:
            if abs(a[0]) >= 1000:
                if abs(a[1]) >= 1000:
                    return '\cfrac{\\nombre{%s}}{\\nombre{%s}}' % a
                else:
                    return '\cfrac{\\nombre{%s}}{%s}' % a
            elif abs(a[1]) >= 1000:
                return '\cfrac{%s}{\\nombre{%s}}' % a
            else:
                return '\cfrac{%s}{%s}' % a


def OOo_frac(a):  # renvoie l'ecriture au format OOo de la fraction a
    if not isinstance(a, tuple):
        return ''
    else:
        if a[1] == 1:
            return a[0]
        else:
            return '{{%s} over {%s}}' % a


def tex_decomp_prod(a):  # renvoie l'ecriture au format tex de la decomposition d'un produit
    if not isinstance(a[2], tuple):  # pas de decomposition possible
        return ''
    elif a[2][0] == 1:

        # decomposition que du denominateur

        b = (a[0][0], a[0][1], a[2][1], a[1][0], a[2][1], a[1][1])
        return '\cfrac{%s}{%s\\times\\bcancel{%s}}\\times\\cfrac{%s\\times\\bcancel{%s}}{%s}' % \
            b
    elif a[2][1] == 1:
        b = (a[0][0], a[2][0], a[0][1], a[1][0], a[1][1], a[2][0])  # decomposition que du numerateur
        return '\cfrac{%s\\times\\cancel{%s}}{%s}\\times\\cfrac{%s}{%s\\times\\cancel{%s}}' % \
            b
    else:

        # decomposition du numerateur et du denominateur

        b = (a[0][0], a[2][0], a[0][1], a[2][1], a[1][0], a[2][1], a[1][1],
             a[2][0])
        return '\\cfrac{%s\\times\\cancel{%s}}{%s\\times\\bcancel{%s}}\\times\\cfrac{%s\\times\\bcancel{%s}}{%s\\times\\cancel{%s}}' % \
            b


def OOo_decomp_prod(a):  # renvoie l'ecriture au format OOo de la decomposition du produit
    if a[2][0] == 1:
        if a[2][1] == 1:  # pas de decomposition
            b = (a[0][0], a[2][1], a[1][0], a[1][1])
            return '{{%s}over{%s}}times{{%s}over{%s}}' % b
        else:

            # decomposition que du denominateur

            b = (a[0][0], a[0][1], a[2][1], a[1][0], a[2][1], a[1][1])
            return '{{%s}over{{%s}times overstrike{%s}}}times{{{%s}times overstrike{%s}}over{%s}}' % \
                b
    elif a[2][1] == 1:
        b = (a[0][0], a[2][0], a[0][1], a[1][0], a[1][1], a[2][0])  # decomposition que du numerateur
        return '{{{%s}times overstrike{%s}}over{%s}}times{{%s}over{{%s}times overstrike{%s}}}' % \
            b
    else:

        # decomposition du numerateur et du denominateur

        b = (a[0][0], a[2][0], a[0][1], a[2][1], a[1][0], a[2][1], a[1][1],
             a[2][0])
        return '{{{%s}times overstrike{%s}}over{{%s}times overstrike{%s}}}times{{{%s}times overstrike{%s}}over{{%s}times overstrike{%s}}}' % \
            b


def tex_den_com0(a, b, c, sgn):  # renvoie l'ecriture au format tex de la mise au meme denominateur des fraction a et b
    if not isinstance(c, tuple):  # les deux fractions ont deja le meme denominateur
        return ''
    else:
        (sgn1, sgn2) = (signe(a[1]), signe(b[1]))
        if c[0] == 1:
            if c[1] == 1:
                d = (a[0] * sgn1, a[1] * sgn1, b[0] * sgn2, b[1] * sgn2)
                return ''
            else:
                d = (a[0] * sgn1, a[1] * sgn1, sgn, b[0] * sgn2, c[1], b[1] * 
                     sgn2, c[1])
                return '\dfrac{%s}{%s}%s\\dfrac{%s_{\\times %s}}{%s_{\\times %s}}' % \
                    d
        elif c[1] == 1:
            d = (a[0] * sgn1, c[0], a[1] * sgn1, c[0], sgn, b[0] * sgn2,
                 b[1] * sgn2)
            return '\dfrac{%s_{\\times %s}}{%s_{\\times %s}}%s\\dfrac{%s}{%s}' % \
                d
        else:
            d = (
                a[0] * sgn1,
                c[0],
                a[1] * sgn1,
                c[0],
                sgn,
                b[0] * sgn2,
                c[1],
                b[1] * sgn2,
                c[1],
                )
            return '\dfrac{%s_{\\times %s}}{%s_{\\times %s}}%s\\dfrac{%s_{\\times %s}}{%s_{\\times %s}}' % \
                d


def tex_den_com1(a, sgn):  # renvoie l'ecriture au format tex de la somme des fractions au meme denominateur
    if not isinstance(a, tuple):  # les deux fractions ont deja le meme denominateur
        return ''
    else:
        (sgn1, sgn2) = (signe(a[0][1]), signe(a[1][1]))
        b = (a[0][0] * sgn1, a[0][1] * sgn1, sgn, a[1][0] * sgn2, a[1][1] * 
             sgn2)
        return '\dfrac{%s}{%s}%s\\dfrac{%s}{%s}' % b


def tex_somme_prod(valeurs, exo, cor):  # calcul du type a+b*c, d contenant (+,*)
    (a, b, c, d) = (valeurs[0], valeurs[1], valeurs[2], valeurs[3])
    if d[1] == '*':
        exo.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_frac(b) + '\\times' + 
                  tex_frac(c) + '\\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_frac(b) + '\\times' + 
                  tex_frac(c) + '\\] ')
        e = decomp_prod(b, c)
        if e[2] != '':
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_decomp_prod(decomp_prod(b,
                      c)) + '\\] ')
    else:
        exo.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_frac(b) + '\\div' + 
                  tex_frac(c) + '\\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_frac(b) + '\\div' + 
                  tex_frac(c) + '\\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_frac(b) + '\\times' + 
                  tex_frac(inverse(c)) + '\\] ')
        e = decomp_prod(b, inverse(c))
        if e[2] != '':
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_decomp_prod(decomp_prod(b,
                      inverse(c))) + '\\] ')
    f = produit(e[0], e[1])
    cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + d[0] + tex_frac(f) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_den_com0(a, f, den_com0(a, f), d[0]) + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_den_com1(den_com1(a, f), d[0]) + '\\] ')
    g = somme(a, f, d[0])
    if isinstance(simplifie(g), tuple):
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(g) + '\\] ')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_frac(simplifie(g)) + '} \\] ')
    else:
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_frac(g) + '} \\] ')


def tex_prod_parenth(valeurs, exo, cor):  # calcul du type a*(b+c), d contenant (*,+)
    (a, b, c, d) = (valeurs[0], valeurs[1], valeurs[2], valeurs[3])
    if d[0] == '*':
        exo.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times\\left(' + tex_frac(b) + d[1] + 
                  tex_frac(c) + '\\right) \\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times\\left(' + tex_frac(b) + d[1] + 
                  tex_frac(c) + '\\right) \\] ')
        if isinstance(den_com0(b, c), tuple):
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times\\left(' + tex_den_com0(b,
                      c, den_com0(b, c), d[1]) + '\\right) \\] ')
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times\\left(' + tex_den_com1(den_com1(b,
                      c), d[1]) + '\\right) \\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times' + tex_frac(somme(b, c, d[1])) + '\\] ')
        if isinstance(simplifie(somme(b, c, d[1])), tuple):
            e = simplifie(somme(b, c, d[1]))
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times' + tex_frac(e) + '\\] ')
        else:
            e = somme(b, c, d[1])
    else:
        exo.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\div\\left(' + tex_frac(b) + d[1] + 
                  tex_frac(c) + '\\right) \\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\div\\left(' + tex_frac(b) + d[1] + 
                  tex_frac(c) + '\\right) \\] ')
        if isinstance(den_com0(b, c), tuple):
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\div\\left(' + tex_den_com0(b,
                      c, den_com0(b, c), d[1]) + '\\right) \\] ')
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\div\\left(' + tex_den_com1(den_com1(b,
                      c), d[1]) + '\\right) \\] ')
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\div' + tex_frac(somme(b, c, d[1])) + '\\] ')
        if isinstance(simplifie(somme(b, c, d[1])), tuple):
            e = simplifie(inverse(somme(b, c, d[1])))
            cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\div' + tex_frac(inverse(e)) + '\\] ')
        else:
            e = inverse(somme(b, c, d[1]))
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(a) + '\\times' + tex_frac(e) + '\\] ')
    f = decomp_prod(a, e)
    cor.append(u'\\[ \\thenocalcul = ' + tex_decomp_prod(f) + '\\] ')
    if isinstance(simplifie(produit(f[0], f[1])), tuple):
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(produit(f[0], f[1])) + '\\] ')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_frac(simplifie(produit(f[0], f[1]))) + '} \\] ')
    else:
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_frac(produit(f[0], f[1])) + '} \\] ')


def tex_quotient_frac(valeurs, exo, cor):  # effectue le quotient {a+b}/{c+d}, e contenant (+,+)
    (a, b, c, d, e) = (valeurs[0], valeurs[1], valeurs[2], valeurs[3],
                       valeurs[4])
    exo.append(u'\\[ \\thenocalcul = \cfrac{' + tex_frac(a) + e[0] + tex_frac(b) + '}{' + 
              tex_frac(c) + e[1] + tex_frac(d) + '} \\] ')
    cor.append(u'\\[ \\thenocalcul =  \cfrac{' + tex_frac(a) + e[0] + tex_frac(b) + '}{' + 
              tex_frac(c) + e[1] + tex_frac(d) + '} \\] ')
    cor.append(u'\\[ \\thenocalcul = \cfrac{' + tex_den_com0(a, b, den_com0(a, b), e[0]) + 
              '}{' + tex_den_com0(c, d, den_com0(c, d), e[1]) + '} \\] ')
    cor.append(u'\\[ \\thenocalcul = \cfrac{' + tex_den_com1(den_com1(a, b), e[0]) + '}{' + 
              tex_den_com1(den_com1(c, d), e[1]) + '} \\] ')

    # ecrit_tex(f1,'\\cfrac{'+tex_frac(somme(a,b,e[0]))+'}{'+tex_frac(somme(c,d,e[1]))+'}')

    cor.append(u'\\[ \\thenocalcul = ' + tex_frac(somme(a, b, e[0])) + '\\div' + tex_frac(somme(c,
              d, e[1])) + ' \\] ')
    cor.append(u'\\[ \\thenocalcul = ' + tex_frac(somme(a, b, e[0])) + '\\times' + tex_frac(inverse(somme(c,
              d, e[1]))) + ' \\] ')
    f = decomp_prod(somme(a, b, e[0]), inverse(somme(c, d, e[1])))
    cor.append(u'\\[ \\thenocalcul = ' + tex_decomp_prod(f) + ' \\] ')
    if isinstance(simplifie(produit(f[0], f[1])), tuple):
        cor.append(u'\\[ \\thenocalcul = ' + tex_frac(produit(f[0], f[1])) + ' \\] ')
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_frac(simplifie(produit(f[0], f[1]))) + '} \\] ')
    else:
        cor.append(u'\\[ \\boxed{\\thenocalcul = ' + tex_frac(produit(f[0], f[1])) + '} \\] ')


def valeurs_somme_prod():  # cree 3 fractions et un tuple de signes (+,*)
    while True:
        (base1, base2) = (valeur_alea(-13, 13), valeur_alea(-13, 13))
        lepgcd = pgcd(base1, base2)
        (base1, base2) = (base1 // lepgcd, base2 // lepgcd)
        if base1 != 1 and base2 != 1:
            break
    (n2, d2) = (base1 * valeur_alea(-10, 10), abs(base2 * valeur_alea(2,
                10)))
    lepgcd = pgcd(n2, d2)
    (n2, d2) = (n2 // lepgcd, d2 // lepgcd)
    (n3, d3) = (base2 * valeur_alea(-10, 10), abs(base1 * valeur_alea(2,
                10)))
    lepgcd = pgcd(n3, d3)
    (n3, d3) = (n3 // lepgcd, d3 // lepgcd)
    (n1, d1) = (base1 * valeur_alea(-10, 10), abs(pgcd(d2, base2 * 
                valeur_alea(2, 10))))
    lepgcd = pgcd(n1, d1)
    (n1, d1) = (n1 // lepgcd, d1 // lepgcd)
    if randrange(2) == 0:
        s1 = '+'
    else:
        s1 = '-'
    if randrange(2) == 0:
        s2 = '*'
    else:
        s2 = ':'
    if s2 == '*':
        return ((n1, d1), (n2, d2), (n3, d3), (s1, s2))
    else:
        return ((n1, d1), (n2, d2), (d3, n3), (s1, s2))


def valeurs_prod_parenth():  # cree 3 fractions et un tuple de signes (*,+)
    while True:
        (base1, base2) = (valeur_alea(2, 13), valeur_alea(2, 13))
        lepgcd = pgcd(base1, base2)
        (base1, base2) = (base1 // lepgcd, base2 // lepgcd)
        if base1 != 1 and base2 != 1:
            break
    while True:
        n2 = valeur_alea(-13, 13)
        lepgcd = pgcd(n2, base1)
        if lepgcd == 1:
            break
    while True:
        n3 = valeur_alea(-13, 13)
        lepgcd = pgcd(n3, base2)
        if lepgcd == 1:
            break
    while True:
        (n1, d1) = (valeur_alea(-10, 10), valeur_alea(2, 10))
        lepgcd = pgcd(n1, d1)
        if lepgcd != n1 and lepgcd != d1:
            break
    (n1, d1) = (n1 // lepgcd, d1 // lepgcd)
    if randrange(2) == 0:
        s1 = '*'
    else:
        s1 = ':'
    if randrange(2) == 0:
        s2 = '+'
    else:
        s2 = '-'
    return ((n1, d1), (n2, base1), (n3, base2), (s1, s2))


def valeurs_quotient_frac():  # cree 4 fractions et un tuple de signes (+,+)
    while True:
        (n1, d1) = (valeur_alea(-10, 10), valeur_alea(2, 10))
        lepgcd = pgcd(n1, d1)
        if lepgcd != n1 and lepgcd != d1:
            break
    (n1, d1) = (n1 // lepgcd, d1 // lepgcd)
    while True:
        (n3, d3) = (valeur_alea(-10, 10), valeur_alea(2, 10))
        lepgcd = pgcd(n3, d3)
        if lepgcd != n3 and lepgcd != d3:
            break
    (n3, d3) = (n3 // lepgcd, d3 // lepgcd)
    (n2, n4) = (valeur_alea(1, 10), valeur_alea(1, 10))
    if randrange(2) == 0:
        s1 = '+'
    else:
        s1 = '-'
    if randrange(2) == 0:
        s2 = '+'
    else:
        s2 = '-'
    return ((n1, d1), (n2, 1), (n3, d3), (n4, 1), (s1, s2))


def _tex_fractions():
    nb_exos = 3
    tex_exos = (tex_somme_prod,
                tex_prod_parenth,
                tex_quotient_frac)
    valeurs_exos = (valeurs_somme_prod,
                    valeurs_prod_parenth,
                    valeurs_quotient_frac)
    ordre_exos = [i for i in range(nb_exos)]
    exo = ['''\n\\exercice''', u"Calculer les expressions suivantes et donner le résultat sous la forme d'une fraction irréductible."]
    exo.append('\\begin{multicols}{3}\\noindent')
    cor = ['''\\exercice*''', u"Calculer les expressions suivantes et donner le résultat sous la forme d'une fraction irréductible."]
    cor.append('\\begin{multicols}{3}\\noindent')
    for i in range(nb_exos):
        a = randrange(nb_exos - i)
        tex_exos[ordre_exos[a]](valeurs_exos[ordre_exos.pop(a)](), exo, cor)
        if i < nb_exos - 1:
            exo.append('\\columnbreak\\stepcounter{nocalcul}')
            cor.append('\\columnbreak\\stepcounter{nocalcul}')
        else:
            exo.append('\\end{multicols}')
            cor.append('\\end{multicols}')
    return (exo, cor)

class tex_fractions(LegacyExercise):
    """Fractions"""

    tags = ["Troisième"]
    function = _tex_fractions



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from random import randint, shuffle
from pyromaths.ex import LegacyExercise

from pyromaths.outils.Affichage import decimaux
from pyromaths.outils.Arithmetique import pgcd, factoriseTex, carrerise

premiers = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61,
            67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139,
            149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223,
            227, 229, 233, 239, 241, 251, 257, 263, 269, 271, 277, 281, 283, 293,
            307, 311, 313, 317, 331, 337, 347, 349, 353, 359, 367, 373, 379, 383,
            389, 397, 401, 409, 419, 421, 431, 433, 439, 443, 449, 457, 461, 463,
            467, 479, 487, 491, 499, 503, 509, 521, 523, 541, 547, 557, 563, 569,
            571, 577, 587, 593, 599, 601, 607, 613, 617, 619, 631, 641, 643, 647,
            653, 659, 661, 673, 677, 683, 691, 701, 709, 719, 727, 733, 739, 743,
            751, 757, 761, 769, 773, 787, 797, 809, 811, 821, 823, 827, 829, 839,
            853, 857, 859, 863, 877, 881, 883, 887, 907, 911, 919, 929, 937, 941,
            947, 953, 967, 971, 977, 983, 991, 997]  # # Liste de longueur 168


def _Arithmetique():
    """Exercice de décomposition de nombres en facteurs premiers, puis de
    recherche du PGCD et du PPCM, et d'applications aux fractions"""

    # ## Question 1
    exo = ["\\exercice", '\\begin{enumerate}', u"\\item Donner la décomposition" +
           u" en facteurs premiers des nombres suivants, et préciser quand il" +
           u" s\'agit d\'un nombre premier :\\par"]
    cor = ["\\exercice*", '\\begin{enumerate}', u"\\item Donner la décomposition"
           + u" en facteurs premiers des nombres suivants, et préciser quand il"
           + u" s\'agit d\'un nombre premier :\\par"]

    prime = premiers[randint(10, 167)]

    fauxpgcd = randint(1, 101)
    fauxpgcdfactor = factoriseTex(fauxpgcd)[0]

    complementaires = [randint(6, 50), randint(6, 50)]
    pgcdcompl = pgcd(complementaires[0], complementaires[1])
    pgcdcomplfact = factoriseTex(pgcdcompl)[0]

    if pgcdcompl != 1:
        facteurs = pgcdcomplfact + fauxpgcdfactor
        facteurs.sort()
    else:
        facteurs = fauxpgcdfactor

    autresnombres = [randint(51, 999), randint(51, 999)]

    listenombres = [prime, complementaires[0] * fauxpgcd, complementaires[1] *
                    fauxpgcd, ] + autresnombres
    melange = [prime, complementaires[0] * fauxpgcd, complementaires[1] *
               fauxpgcd, ] + autresnombres
    shuffle(melange)

    exo.append(decimaux(melange[0]) + " ; " + decimaux(melange[1]) + " ; " +
               decimaux(melange[2]) + " ; " + decimaux(melange[3]) + " ; " +
               decimaux(melange[4]) + " ;")

    cor.append("\\begin{multicols}{2}")

    for i in range(5):
        cor += factoriseTex(melange[i])[1]

    cor.append("\\end{multicols}")

    # ## Question 2
    exo.append(u'\\item En déduire le PGCD et le PPCM des nombres ' +
               decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
               ".")
    cor.append(u'\\item En déduire le PGCD et le PPCM des nombres ' +
               decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
               ".\\par")

    cor.append(u"D'après la question 1), on sait que les nombres " +
               decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
               " ont comme facteurs premiers communs : ")

    for j in range(len(facteurs)):
        if j == 0:
            temp = "$"
        if j != len(facteurs) - 1:
            temp += decimaux(facteurs[j]) + " , "
        else:
            temp += decimaux(facteurs[j]) + "$.\\par"
    cor.append(temp)

    cor.append(u"On en déduit que le PGCD des nombres " +
               decimaux(listenombres[1]) + " et " + decimaux(listenombres[2]) +
               " est : ")
    temp = "$"
    if len(facteurs) > 1:
        for j in range(len(facteurs)):
            if j != len(facteurs) - 1:
                temp += decimaux(facteurs[j]) + " \\times "
            else:
                temp += decimaux(facteurs[j]) + " = "

    temp += decimaux(fauxpgcd * pgcdcompl) + ".$\\par"
    cor.append(temp)

    vraippcm = (listenombres[1] * listenombres[2]) // (fauxpgcd * pgcdcompl)

    if (listenombres[1] % listenombres[2] == 0):
        cor.append(decimaux(listenombres[1]) + u" est un multiple de " +
                   decimaux(listenombres[2]) + u", donc leur PPCM est directement "
                   + decimaux(listenombres[1]) + ".")
    elif (listenombres[2] % listenombres[1] == 0):
        cor.append(decimaux(listenombres[2]) + u" est un multiple de " +
                   decimaux(listenombres[1]) + u", donc leur PPCM est directement " +
                   decimaux(listenombres[2]) + ".")
    else:
        cor.append(u"Il existe plusieurs méthodes pour calculer le PPCM de " +
                   decimaux(listenombres[1]) + " et de " + decimaux(listenombres[2]) +
                   ".\\par")
        cor.append(u"En voici deux :")
        cor.append("\\begin{enumerate}")

        cor.append(u"\\item On peut simplement utiliser la formule :")
        cor.append(u"$a \\times b = PGCD(a;~b) \\times PPCM(a;~b)$.\\par")
        cor.append(u"Donc : $PPCM(" + decimaux(listenombres[1]) + ";~" +
                   decimaux(listenombres[2]) + ") = " + "\\dfrac{" +
                   decimaux(listenombres[1]) + "\\times" + decimaux(listenombres[2]) + "}{"
                   + decimaux(fauxpgcd * pgcdcompl) + "} = " + decimaux(vraippcm) +
                   ".$")

        cor.append(u"\\item On peut aussi multiplier un nombre par les \"facteurs "
                   + u"complémentaires\" de l'autre.\n" + u"Ces \"facteurs " +
                   u"complémentaires\" sont les facteurs qui complètent le PGCD pour " +
                   u"former le nombre.\\par")

        temp = u"Comme $PGCD(" + decimaux(listenombres[1]) + ";~" + \
               decimaux(listenombres[2]) + ") = " + decimaux(fauxpgcd * pgcdcompl)

        if len(facteurs) > 1:
            temp += " = "
            for j in range(len(facteurs)):
                if j != len(facteurs) - 1:
                    temp += decimaux(facteurs[j]) + " \\times "
                else:
                    temp += decimaux(facteurs[j])

        factornb1 = factoriseTex(listenombres[1])[0]

        if len(factornb1) > 1:
            textcompl = u"$, alors les \"facteurs complémentaires\" de $"
        else:
            textcompl = u"$, alors le \"facteur complémentaire\" de $"

        temp += textcompl + decimaux(listenombres[1]) + " = "

        for j in range(len(factornb1)):
            if j != len(factornb1) - 1:
                temp += decimaux(factornb1[j]) + " \\times "
            else:
                temp += decimaux(factornb1[j])

        factcompl = factoriseTex(listenombres[1] // (fauxpgcd * pgcdcompl))[0]

        if len(factcompl) == 1:
            temp += u"$ est : "
        else:
            temp += u"$ sont : "
        for j in range(len(factcompl)):
            if j != len(factcompl) - 1:
                temp += decimaux(factcompl[j]) + " , "
            else:
                temp += decimaux(factcompl[j]) + ".\n"
        temp += u"On en déduit que $PPCM(" + decimaux(listenombres[1]) + ";~" + \
                decimaux(listenombres[2]) + ") = " + decimaux(listenombres[2]) + \
                " \\times "

        for j in range(len(factcompl)):
            if j != len(factcompl) - 1:
                temp += decimaux(factcompl[j]) + " \\times "
            else:
                temp += decimaux(factcompl[j]) + " = "
        temp += decimaux(vraippcm) + ".$"
        cor.append(temp)
        cor.append("\\end{enumerate}")

    # ## Question 3

    exo.append(u"\\item Quel est le plus petit nombre par lequel il faut " +
               u"multiplier " + decimaux(autresnombres[0]) +
               u" pour obtenir un carré parfait ?")

    cor.append(u" \\item Pour obtenir un carré parfait, il faut que sa " +
               u"décomposition en facteurs premiers ne contienne que des facteurs "
               + u"apparaissant un nombre pair de fois. D'après la question 1, " +
               u"la décomposition en facteurs premiers de "
               + decimaux(autresnombres[0]))

    decompautre = factoriseTex(autresnombres[0])[1]

    if len(decompautre) == 1:
        cor.append(u" est lui-même, car c'est un nombre premier.")
    else:
        cor.append(" est : \\par\n$" + decimaux(autresnombres[0]) + " = " +
                   decompautre[-2][5:-2] + ".$\\par")

    cor.append(u"Il faut donc encore multiplier ce nombre par ")

    carre = carrerise(autresnombres[0])
    factsup = factoriseTex(carre)[0]

    if len(factsup) == 1:
        cor.append(" le facteur ")
    else:
        cor.append(" les facteurs ")

    for j in range(len(factsup)):
        if (j != len(factsup) - 1) and (j != len(factsup) - 2):
            cor.append(decimaux(factsup[j]) + " , ")
        elif (j == len(factsup) - 2):
            cor.append(decimaux(factsup[j]) + " et ")
        else:
            cor.append(decimaux(factsup[j]) + ".\\par")

    cor.append(u"Le nombre cherché est par conséquent " + decimaux(carre) +
               u" et le carré parfait obtenu est " + decimaux(carre *
                                                              autresnombres[0]) + ".")

    # ## Question 4
    exo.append(u"\\item Rendre la fraction $\\dfrac{" + decimaux(listenombres[1])
               + "}{" + decimaux(listenombres[2]) + u"}$ irréductible.")

    cor.append(u"\\item Le moyen le plus rapide de simplifier cette fraction est"
               + u"de diviser le numérateur et le dénominateur par leur PGCD." +
               u" D'après la question 2),  PGCD(" + decimaux(listenombres[1]) + ";~"
               + decimaux(listenombres[2]) + ") = "
               + decimaux(fauxpgcd * pgcdcompl) + ", donc on obtient :\\par")
    cor.append(u"$\dfrac{" + decimaux(listenombres[1]) + "{\\scriptstyle \\div " +
               decimaux(fauxpgcd * pgcdcompl) + "}}{" + decimaux(listenombres[2]) +
               "{\\scriptstyle \\div " + decimaux(fauxpgcd * pgcdcompl) +
               "}} = \dfrac{" + decimaux(listenombres[1] // (fauxpgcd * pgcdcompl)) +
               "}{" + decimaux(listenombres[2] // (fauxpgcd * pgcdcompl)) + "}.$")

    # ## Question 5

    num = [randint(6, 50), randint(6, 50)]
    exo.append(u"\\item Calculer $\\dfrac{" + decimaux(num[0]) + "}{" +
               decimaux(listenombres[1]) + "} + \\dfrac{" + decimaux(num[1]) + "}{" +
               decimaux(listenombres[2]) + "}$.")

    mult1 = vraippcm // listenombres[1]
    mult2 = vraippcm // listenombres[2]

    num1 = mult1 * num[0]
    num2 = mult2 * num[1]

    simplfin = pgcd(num1 + num2, vraippcm)

    if simplfin != 1:
        simpl = "{\\scriptstyle \\div " + decimaux(simplfin) + "}"
        result = " = \\dfrac{" + decimaux((num1 + num2) // simplfin) + "}{" + \
                 decimaux((vraippcm) // simplfin) + "}"
    else:
        simpl = ""
        result = ""

    cor.append(u"\\item Il faut mettre les fractions au même dénominateur. Grâce"
               + u"à la question 2), nous avons déjà un dénominateur commun : " +
               u"le PPCM des nombres " + decimaux(listenombres[1]) + " et " +
               decimaux(listenombres[2]) + u", qui est par définition le plus petit"
               + u"multiple commun de ces deux nombres.\\par")
    cor.append(u"$\\dfrac{" + decimaux(num[0]) + "{\\scriptstyle \\times " +
               decimaux(mult1) + "}}{" + decimaux(listenombres[1]) +
               "{\\scriptstyle \\times " + decimaux(mult1) + "}} + \\dfrac{" +
               decimaux(num[1]) + "{\\scriptstyle \\times " + decimaux(mult2) + "}}{"
               + decimaux(listenombres[2]) + "{\\scriptstyle \\times " +
               decimaux(mult2) + "}} = \\dfrac{" + decimaux(num1) + "}{" +
               decimaux(vraippcm) + "} + \\dfrac{" + decimaux(num2) + "}{" +
               decimaux(vraippcm) + "} = \\dfrac{" + decimaux(num1 + num2) + simpl +
               "}{" + decimaux(vraippcm) + simpl + "}" + result + ".$")

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")

    return (exo, cor)

class Arithmetique(LegacyExercise):
    """Arithmétique"""

    tags = ["Troisième"]
    function = _Arithmetique
